@@ -3,10 +3,10 @@
 from __future__ import annotations
 
 import functools
+import math
 import operator
 import os
 from collections import defaultdict
-from math import exp, sqrt
 from typing import TYPE_CHECKING
 
 import numpy as np
@@ -55,8 +55,8 @@ def calculate_bv_sum(site, nn_list, scale_factor=1.0):
             r2 = BV_PARAMS[el2]["r"]
             c1 = BV_PARAMS[el1]["c"]
             c2 = BV_PARAMS[el2]["c"]
-            R = r1 + r2 - r1 * r2 * (sqrt(c1) - sqrt(c2)) ** 2 / (c1 * r1 + c2 * r2)
-            vij = exp((R - nn.nn_distance * scale_factor) / 0.31)
+            R = r1 + r2 - r1 * r2 * (math.sqrt(c1) - math.sqrt(c2)) ** 2 / (c1 * r1 + c2 * r2)
+            vij = math.exp((R - nn.nn_distance * scale_factor) / 0.31)
             bv_sum += vij * (1 if el1.X < el2.X else -1)
     return bv_sum
 
@@ -91,8 +91,8 @@ def calculate_bv_sum_unordered(site, nn_list, scale_factor=1):
                     r2 = BV_PARAMS[el2]["r"]
                     c1 = BV_PARAMS[el1]["c"]
                     c2 = BV_PARAMS[el2]["c"]
-                    R = r1 + r2 - r1 * r2 * (sqrt(c1) - sqrt(c2)) ** 2 / (c1 * r1 + c2 * r2)
-                    vij = exp((R - nn.nn_distance * scale_factor) / 0.31)
+                    R = r1 + r2 - r1 * r2 * (math.sqrt(c1) - math.sqrt(c2)) ** 2 / (c1 * r1 + c2 * r2)
+                    vij = math.exp((R - nn.nn_distance * scale_factor) / 0.31)
                     bv_sum += occu1 * occu2 * vij * (1 if el1.X < el2.X else -1)
     return bv_sum
 
@@ -173,7 +173,7 @@ class BVAnalyzer:
                 sigma = data["std"]
                 # Calculate posterior probability. Note that constant
                 # factors are ignored. They have no effect on the results.
-                prob[sp.oxi_state] = exp(-((bv_sum - u) ** 2) / 2 / (sigma**2)) / sigma * PRIOR_PROB[sp]
+                prob[sp.oxi_state] = math.exp(-((bv_sum - u) ** 2) / 2 / (sigma**2)) / sigma * PRIOR_PROB[sp]
         # Normalize the probabilities
         try:
             prob = {k: v / sum(prob.values()) for k, v in prob.items()}
@@ -194,7 +194,7 @@ class BVAnalyzer:
                     sigma = data["std"]
                     # Calculate posterior probability. Note that constant
                     # factors are ignored. They have no effect on the results.
-                    prob[el][sp.oxi_state] = exp(-((bv_sum - u) ** 2) / 2 / (sigma**2)) / sigma * PRIOR_PROB[sp]
+                    prob[el][sp.oxi_state] = math.exp(-((bv_sum - u) ** 2) / 2 / (sigma**2)) / sigma * PRIOR_PROB[sp]
             # Normalize the probabilities
             try:
                 prob[el] = {k: v / sum(prob[el].values()) for k, v in prob[el].items()}
@@ -263,7 +263,7 @@ class BVAnalyzer:
                     # Retain probabilities that are at least 1/100 of highest prob.
                     filtered = list(
                         filter(
-                            lambda v: prob[elem.symbol][v] > 0.001 * prob[elem.symbol][val[0]],
+                            lambda v: prob[elem.symbol][v] > 1e-3 * prob[elem.symbol][val[0]],
                             val,
                         )
                     )
@@ -414,9 +414,7 @@ class BVAnalyzer:
 
                 return [int(assigned[site]) for site in structure]
             assigned = {}
-            new_best_vset = []
-            for _ in equi_sites:
-                new_best_vset.append([])
+            new_best_vset = [[] for _ in equi_sites]
             for ival, val in enumerate(self._best_vset):
                 new_best_vset[attrib[ival]].append(val)
             for val, sites in zip(new_best_vset, equi_sites, strict=True):
