@@ -22,6 +22,8 @@ from pymatgen.analysis.chemenv.utils.math_utils import get_linearly_independent_
 if TYPE_CHECKING:
     from typing_extensions import Self
 
+logger = logging.getLogger(__name__)
+
 
 def draw_network(env_graph, pos, ax, sg=None, periodicity_vectors=None):
     """Draw network of environments in a matplotlib figure axes.
@@ -137,8 +139,8 @@ def make_supergraph(graph, multiplicity, periodicity_vectors):
         nx.MultiGraph: Super graph of the environments.
     """
     super_graph = nx.MultiGraph()
-    print("periodicity vectors :")
-    print(periodicity_vectors)
+    logger.info("periodicity vectors :")
+    logger.info(periodicity_vectors)
     if isinstance(multiplicity, int) or len(multiplicity) == 1:
         mult = multiplicity if isinstance(multiplicity, int) else multiplicity[0]
         nodes = graph.nodes(data=True)
@@ -148,7 +150,7 @@ def make_supergraph(graph, multiplicity, periodicity_vectors):
         connecting_edges = []
         other_edges = []
         for n1, n2, key, data in edges:
-            print(n1, n2, key, data)
+            logger.info(n1, n2, key, data)
             if np.all(np.array(data["delta"]) == np.array(periodicity_vectors[0])):
                 connecting_edges.append((n1, n2, key, data))
             elif np.all(np.array(data["delta"]) == -np.array(periodicity_vectors[0])):
@@ -159,8 +161,8 @@ def make_supergraph(graph, multiplicity, periodicity_vectors):
                 connecting_edges.append((n1, n2, key, new_data))
             else:
                 if not np.all(np.array(data["delta"]) == 0):
-                    print(
-                        "delta not equal to periodicity nor 0 ... : ",
+                    logger.info(
+                        "delta not equal to periodicity nor 0 ... : %s %s %s %s %s",
                         n1,
                         n2,
                         key,
@@ -449,8 +451,7 @@ class ConnectedComponent(MSONable):
                         this_path_deltas_new = []
                         for edge_data in self._connected_subgraph[node1][node2].values():
                             delta = get_delta(node1, node2, edge_data)
-                            for current_delta in this_path_deltas:
-                                this_path_deltas_new.append(current_delta + delta)
+                            this_path_deltas_new.extend(current_delta + delta for current_delta in this_path_deltas)
                         this_path_deltas = this_path_deltas_new
                     this_node_cell_img_vectors.extend(this_path_deltas)
                     this_node_cell_img_vectors = get_linearly_independent_vectors(this_node_cell_img_vectors)
@@ -482,11 +483,10 @@ class ConnectedComponent(MSONable):
             cyc.append(cyc[0])
             this_cycle_deltas = [np.zeros(3, int)]
             for node1, node2 in [(node1, cyc[inode1 + 1]) for inode1, node1 in enumerate(cyc[:-1])]:
-                this_cycle_deltas_new = []
+                this_cycle_deltas_new: list = []
                 for edge_data in self._connected_subgraph[node1][node2].values():
                     delta = get_delta(node1, node2, edge_data)
-                    for current_delta in this_cycle_deltas:
-                        this_cycle_deltas_new.append(current_delta + delta)
+                    this_cycle_deltas_new.extend(current_delta + delta for current_delta in this_cycle_deltas)
                 this_cycle_deltas = this_cycle_deltas_new
             all_deltas.extend(this_cycle_deltas)
             all_deltas = get_linearly_independent_vectors(all_deltas)
@@ -650,7 +650,7 @@ class ConnectedComponent(MSONable):
         Returns:
             nx.MultiGraph: Elastic centered subgraph.
         """
-        logging.info("In elastic centering")
+        logger.info("In elastic centering")
         # Loop on start_nodes, sometimes some nodes cannot be elastically taken
         # inside the cell if you start from a specific node
         n_test_nodes = 0
@@ -670,17 +670,17 @@ class ConnectedComponent(MSONable):
         tree_level = 0
         while True:
             tree_level += 1
-            logging.debug(f"In tree level {tree_level} ({len(current_nodes)} nodes)")
+            logger.debug(f"In tree level {tree_level} ({len(current_nodes)} nodes)")
             new_current_nodes = []
             # Loop on nodes in this level of the tree
             for node in current_nodes:
                 inode += 1
-                logging.debug(f"  In node #{inode}/{len(current_nodes)} in level {tree_level} ({node})")
+                logger.debug(f"  In node #{inode}/{len(current_nodes)} in level {tree_level} ({node})")
                 node_neighbors = list(tree.neighbors(n=node))
                 node_edges = centered_connected_subgraph.edges(nbunch=[node], data=True, keys=True)
                 # Loop on neighbors of a node (from the tree used)
                 for inode_neighbor, node_neighbor in enumerate(node_neighbors):
-                    logging.debug(
+                    logger.debug(
                         f"    Testing neighbor #{inode_neighbor}/{len(node_neighbors)} ({node_neighbor}) of "
                         f"node #{inode} ({node})"
                     )
@@ -698,22 +698,22 @@ class ConnectedComponent(MSONable):
                             else:
                                 raise ValueError("Should not be here ...")
                             ddeltas.append(thisdelta)
-                    logging.debug(
+                    logger.debug(
                         "        ddeltas : " + ", ".join(f"({', '.join(str(ddd) for ddd in dd)})" for dd in ddeltas)
                     )
                     if ddeltas.count((0, 0, 0)) > 1:
                         raise ValueError("Should not have more than one 000 delta ...")
                     if already_inside:
-                        logging.debug("          Edge inside the cell ... continuing to next neighbor")
+                        logger.debug("          Edge inside the cell ... continuing to next neighbor")
                         continue
-                    logging.debug("          Edge outside the cell ... getting neighbor back inside")
+                    logger.debug("          Edge outside the cell ... getting neighbor back inside")
                     if (0, 0, 0) in ddeltas:
                         ddeltas.remove((0, 0, 0))
                     d_delta = np.array(ddeltas[0], int)
                     node_neighbor_edges = centered_connected_subgraph.edges(
                         nbunch=[node_neighbor], data=True, keys=True
                     )
-                    logging.debug(
+                    logger.debug(
                         f"            Delta image from {node=} to {node_neighbor=} : ({', '.join(map(str, d_delta))})"
                     )
                     # Loop on the edges of this neighbor
@@ -731,7 +731,7 @@ class ConnectedComponent(MSONable):
                                 )
                             else:
                                 raise ValueError("DUHH")
-                            logging.debug(
+                            logger.debug(
                                 f"                  {n1} to node {n2} now has delta "
                                 f"{centered_connected_subgraph[n1][n2][key]['delta']}"
                             )
