@@ -57,7 +57,7 @@ def get_lower_and_upper_f(surface_calculation_options):
             lower_points=lower_points, upper_points=upper_points, degree=degree
         )
     else:
-        raise ValueError(f"Surface calculation of type \"{surface_calculation_options['type']}\" is not implemented")
+        raise ValueError(f'Surface calculation of type "{surface_calculation_options["type"]}" is not implemented')
     return lower_and_upper_functions
 
 
@@ -477,10 +477,8 @@ def changebasis(uu, vv, nn, pps):
         MM[ii, 1] = vv[ii]
         MM[ii, 2] = nn[ii]
     PP = np.linalg.inv(MM)
-    new_pps = []
-    for pp in pps:
-        new_pps.append(matrixTimesVector(PP, pp))
-    return new_pps
+
+    return [matrixTimesVector(PP, pp) for pp in pps]
 
 
 def collinear(p1, p2, p3=None, tolerance=0.25):
@@ -525,8 +523,8 @@ def anticlockwise_sort(pps):
     for ipp, pp in enumerate(pps):
         angles[ipp] = np.arctan2(pp[1], pp[0])
     idx_sorted = np.argsort(angles)
-    for ii in range(len(pps)):
-        new_pps.append(pps[idx_sorted[ii]])
+
+    new_pps.extend(pps[idx_sorted[ii]] for ii in range(len(pps)))
     return new_pps
 
 
