@@ -22,13 +22,13 @@ who outlined many of its possible uses:
 
 from __future__ import annotations
 
-import json
 import warnings
 from functools import lru_cache
 from itertools import groupby
 from typing import TYPE_CHECKING
 
 import numpy as np
+import orjson
 import plotly.express as px
 from monty.json import MSONable
 from plotly.graph_objects import Figure, Mesh3d, Scatter, Scatter3d
@@ -44,8 +44,8 @@ from pymatgen.util.string import htmlify
 if TYPE_CHECKING:
     from pymatgen.entries.computed_entries import ComputedEntry
 
-with open(f"{PKG_DIR}/util/plotly_chempot_layouts.json", encoding="utf-8") as file:
-    plotly_layouts = json.load(file)
+with open(f"{PKG_DIR}/util/plotly_chempot_layouts.json", "rb") as file:
+    plotly_layouts = orjson.loads(file.read())
 
 
 @due.dcite(
