@@ -4,10 +4,10 @@ from __future__ import annotations
 
 import bisect
 import math
+import warnings
 from copy import copy, deepcopy
 from datetime import datetime, timezone
 from typing import TYPE_CHECKING
-from warnings import warn
 
 import numpy as np
 from monty.json import MSONable
@@ -176,9 +176,7 @@ class EwaldSummation(MSONable):
 
         if len(matches) != len(sub_structure):
             output = ["Missing sites."]
-            for site in sub_structure:
-                if site not in matches:
-                    output.append(f"unmatched = {site}")
+            output.extend(f"unmatched = {site}" for site in sub_structure if site not in matches)
             raise ValueError("\n".join(output))
 
         return sum(sum(total_energy_matrix))
@@ -288,7 +286,7 @@ class EwaldSummation(MSONable):
             self._initialized = True
 
         if self._charged:
-            warn("Per atom energies for charged structures not supported in EwaldSummation")
+            warnings.warn("Per atom energies for charged structures not supported in EwaldSummation", stacklevel=2)
         return np.sum(self._recip[:, site_index]) + np.sum(self._real[:, site_index]) + self._point[site_index]
 
     def _calc_ewald_terms(self):
