@@ -3,10 +3,10 @@
 from __future__ import annotations
 
 import itertools
+import math
+import warnings
 from collections import defaultdict
-from math import acos, pi
 from typing import TYPE_CHECKING
-from warnings import warn
 
 import matplotlib.pyplot as plt
 import numpy as np
@@ -133,9 +133,8 @@ class VoronoiAnalyzer:
             if step % step_freq != 0:
                 continue
 
-            v = []
-            for n in range(len(structure)):
-                v.append(str(self.analyze(structure, n=n).view()))
+            v = [str(self.analyze(structure, n=n).view()) for n in range(len(structure))]
+
             for voro in v:
                 if voro in voro_dict:
                     voro_dict[voro] += 1
@@ -249,7 +248,7 @@ class VoronoiConnectivity:
         self.cutoff = cutoff
         self.structure = structure
         recip_vec = np.array(self.structure.lattice.reciprocal_lattice.abc)
-        cutoff_vec = np.ceil(cutoff * recip_vec / (2 * pi))
+        cutoff_vec = np.ceil(cutoff * recip_vec / (2 * np.pi))
         offsets = np.mgrid[
             -cutoff_vec[0] : cutoff_vec[0] + 1,
             -cutoff_vec[1] : cutoff_vec[1] + 1,
@@ -296,7 +295,10 @@ class VoronoiConnectivity:
                 connectivity[atom_j, atom_i, image_i] = val
 
             if -10.101 in vts[v]:
-                warn("Found connectivity with infinite vertex. Cutoff is too low, and results may be incorrect")
+                warnings.warn(
+                    "Found connectivity with infinite vertex. Cutoff is too low, and results may be incorrect",
+                    stacklevel=2,
+                )
         return connectivity
 
     @property
@@ -356,9 +358,9 @@ def solid_angle(center, coords):
         v = -np.dot(cross_products[i], cross_products[i + 1]) / (
             np.linalg.norm(cross_products[i]) * np.linalg.norm(cross_products[i + 1])
         )
-        vals.append(acos(np.clip(v, -1, 1)))
+        vals.append(math.acos(np.clip(v, -1, 1)))
     phi = sum(vals)
-    return phi + (3 - len(radii)) * pi
+    return phi + (3 - len(radii)) * np.pi
 
 
 def get_max_bond_lengths(structure, el_radius_updates=None):
@@ -456,7 +458,7 @@ class OxideType:
         if h_sites_frac_coords:
             dist_matrix = lattice.get_all_distances(o_sites_frac_coords, h_sites_frac_coords)
             if np.any(dist_matrix < relative_cutoff * 0.93):
-                return "hydroxide", int(len(np.where(dist_matrix < relative_cutoff * 0.93)[0]) / 2)
+                return "hydroxide", len(np.where(dist_matrix < relative_cutoff * 0.93)[0]) // 2
         dist_matrix = lattice.get_all_distances(o_sites_frac_coords, o_sites_frac_coords)
         np.fill_diagonal(dist_matrix, 1000)
         is_superoxide = is_peroxide = is_ozonide = False
