@@ -29,6 +29,7 @@ if TYPE_CHECKING:
 
     from pymatgen.entries import Entry
     from pymatgen.entries.computed_entries import ComputedEntry, ComputedStructureEntry
+    from pymatgen.util.typing import SpeciesLike
 
 logger = logging.getLogger(__name__)
 
@@ -130,7 +131,7 @@ def group_entries_by_structure(
         for entry, host in entries_host:
             symm_entries[comparator.get_structure_hash(host)].append((entry, host))
 
-        logging.info(f"Using {ncpus} cpus")
+        logger.info(f"Using {ncpus} cpus")
         manager = mp.Manager()
         groups = manager.list()
         with mp.Pool(ncpus) as pool:
@@ -168,11 +169,9 @@ def group_entries_by_structure(
                 groups,
             )
         )
-    entry_groups = []
-    for g in groups:
-        entry_groups.append(json.loads(g, cls=MontyDecoder))
-    logging.info(f"Finished at {datetime.now(tz=timezone.utc)}")
-    logging.info(f"Took {datetime.now(tz=timezone.utc) - start}")
+    entry_groups = [json.loads(g, cls=MontyDecoder) for g in groups]
+    logger.info(f"Finished at {datetime.now(tz=timezone.utc)}")
+    logger.info(f"Took {datetime.now(tz=timezone.utc) - start}")
     return entry_groups
 
 
@@ -309,7 +308,7 @@ class EntrySet(collections.abc.MutableSet, MSONable):
             latexify_names: Format entry names to be LaTex compatible,
                 e.g. Li_{2}O
         """
-        els: set[Element] = set()
+        els: set[SpeciesLike] = set()
         for entry in self.entries:
             els.update(entry.elements)
         elements = sorted(els, key=lambda a: a.X)
