@@ -9,6 +9,7 @@ https://materialsproject.org/dashboard.
 
 from __future__ import annotations
 
+import gzip
 import itertools
 import json
 import logging
@@ -16,48 +17,89 @@ import os
 import platform
 import sys
 import warnings
+from functools import partial
 from typing import TYPE_CHECKING, NamedTuple
 
+import numpy as np
+import orjson
 import requests
 from monty.json import MontyDecoder
 
-from pymatgen.core import SETTINGS
+from pymatgen.core import SETTINGS, Lattice
 from pymatgen.core import __version__ as PMG_VERSION
+from pymatgen.core.composition import Composition
+from pymatgen.electronic_structure.bandstructure import Kpoint
+from pymatgen.phonon import CompletePhononDos, PhononBandStructureSymmLine
 from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
 
 if TYPE_CHECKING:
-    from collections.abc import Callable
+    from collections.abc import Callable, Sequence
+    from typing import Any
 
-    from mp_api.client import MPRester as _MPResterNew
     from typing_extensions import Self
 
     from pymatgen.core.structure import Structure
     from pymatgen.entries.computed_entries import ComputedStructureEntry
-    from pymatgen.ext.matproj_legacy import _MPResterLegacy
 
 logger = logging.getLogger(__name__)
 
 MP_LOG_FILE = os.path.join(os.path.expanduser("~"), ".mprester.log.yaml")
 
+CHUNK_SIZE = 200
 
-class _MPResterBasic:
+
+class MPRester:
     """
-    This is Pymatgen's own implement of a MPRester that supports the new MP API. If you are getting your API key from
-    the new dashboard of MP, you will need to use this instead of the original MPRester because the new API keys do
-    not work with the old MP API (???!).
+    Pymatgen's implementation of MPRester. Unlike mp-api, this implementation mirrors the exact MP-API end points
+    without modification. You just need to refer to https://api.materialsproject.org/docs and use the field names
+    exactly. No need to deal with strange renames of various fields. Featurity parity is close to 100% with mp-api.
 
-    The reason why this class exists is because it is the belief of the pymatgen maintainers that access to the MP API
-    remains a critical part of pymatgen functionality. However, the new mp-api package is written in a way that
-    prevents us from importing the mp-api package because of cyclic dependencies. It is also the opinion of Pymatgen
-    maintainers that the implementation of mp-api is too heavy duty for most users (few care about document models,
-    etc.). Further, this implementation serves as a simple reference for end developers who want to develop their own
-    interfaces to the MP API via the REST urls.
+    Furthermore, we support both the mp-api as well as a simplified syntax. E.g., to query for a summary, you can use
+    mpr.summary.search(material_ids="mp-1234") or mpr.materials.summary.search(material_ids="mp-1234").
 
-    If you are a power user, feel free to install the mp-api package. All issues regarding that implementation should
-    be directed to the maintainers of that repository and not pymatgen. We will support only issues with regards to
-    our implementation only.
+    If you are a power user that requires some esoteric feature not covered, feel free to install the mp-api package.
+    All issues regarding that implementation should be directed to the maintainers of that repository and not
+    pymatgen. We will support only issues pertaining to our implementation only.
+
+    Attributes:
+    :ivar api_key: API key for authenticating requests to the Materials Project API.
+    :type api_key: str
+    :ivar preamble: Base endpoint URL for the Materials Project API.
+    :type preamble: str
+    :ivar session: HTTP session object for managing API requests.
+    :type session: requests.Session
+    :ivar materials: Placeholder object for dynamically adding endpoints related to materials.
+    :type materials: Any
     """
 
+    MATERIALS_DOCS = (
+        "summary",
+        "core",
+        "elasticity",
+        "phonon",
+        "eos",
+        "similarity",
+        "xas",
+        "grain_boundaries",
+        "electronic_structure",
+        "tasks",
+        "substrates",
+        "surface_properties",
+        "robocrys",
+        "synthesis",
+        "magnetism",
+        "insertion_electrodes",
+        "conversion_electrodes",
+        "oxidation_states",
+        "provenance",
+        "alloys",
+        "absorption",
+        "chemenv",
+        "bonds",
+        "piezoelectric",
+        "dielectric",
+    )
+
     def __init__(self, api_key: str | None = None, include_user_agent: bool = True) -> None:
         """
         Args:
@@ -78,6 +120,12 @@ class _MPResterBasic:
             self.api_key = api_key
         else:
             self.api_key = SETTINGS.get("PMG_MAPI_KEY", "")
+
+        if len(self.api_key) != 32:
+            raise ValueError(
+                "Invalid or old API key. Please obtain an updated API key at https://materialsproject.org/dashboard."
+            )
+
         self.preamble = SETTINGS.get("PMG_MAPI_ENDPOINT", "https://api.materialsproject.org/")
 
         self.session = requests.Session()
@@ -89,17 +137,23 @@ class _MPResterBasic:
             self.session.headers["user-agent"] = f"{pymatgen_info} ({python_info} {platform_info})"
 
         # This is a hack, but it fakes most of the functionality of mp-api's summary.search.
-        class Summary(NamedTuple):
+        class Search(NamedTuple):
             search: Callable
 
-        self.summary = Summary(self.summary_search)
+        class Materials:
+            pass
+
+        self.materials = Materials()
+
+        for doc in MPRester.MATERIALS_DOCS:
+            setattr(self, doc, Search(partial(self.search, doc)))
+            setattr(self.materials, doc, Search(partial(self.search, doc)))
 
     def __getattr__(self, item):
-        if item in ("summary", "materials", "thermo"):
-            raise AttributeError(
-                f"{item} is not an attribute of this implementation of MPRester, which only supports functionality"
-                "used by 80% of users. If you are looking for the full functionality MPRester, pls install the mp-api ."
-            )
+        raise AttributeError(
+            f"{item} is not an attribute of this implementation of MPRester, which only supports functionality "
+            "used by 80% of users. If you are looking for the full functionality MPRester, pls install the mp-api ."
+        )
 
     def __enter__(self) -> Self:
         """Support for "with" context."""
@@ -124,8 +178,8 @@ class _MPResterBasic:
                     response = self.session.post(actual_url, data=payload, verify=True)
                 else:
                     response = self.session.get(actual_url, params=payload, verify=True)
-                if response.status_code in [200, 400]:
-                    data = json.loads(response.text, cls=MontyDecoder) if mp_decode else json.loads(response.text)
+                if response.status_code in {200, 400}:
+                    data = json.loads(response.text, cls=MontyDecoder) if mp_decode else orjson.loads(response.text)
                 else:
                     raise MPRestError(f"REST query returned with error status code {response.status_code}")
                 all_data.extend(data["data"])
@@ -137,24 +191,58 @@ class _MPResterBasic:
                 raise MPRestError(msg)
         return all_data
 
-    def summary_search(self, **kwargs) -> list[dict]:
-        """This function mirrors the mp-api's summary.search functionality.
-
-        Args:
-            **kwargs: This function only takes kwargs. All kwargs that do not start with an underscore are treated as
-                search criteria and those with underscores are treated as params. Example usage:
-                MPRester().summary.search(material_ids="mp-19770,mp-19017", _fields="formula_pretty,energy_above_hull")
+    def search(self, doc, **kwargs) -> list[dict]:
         """
-        criteria = {k: v for k, v in kwargs.items() if not k.startswith("_")}
-        params = [f"{k}={v}" for k, v in kwargs.items() if k.startswith("_") and k != "_fields"]
+        Queries a Materials PI end point doc. A notable difference with the mp-api's implementation is that this uses
+        the web API to do searches. So the keywords follow the actual API spec, which is as it should be. For
+        instance, number of sites is `nsites` and number of elements is `nelements`. The mp-api package has this
+        weird renaming that maps `num_elements` to `nelements` and `num_sites` to  `nsites`.
+
+        Parameters:
+        - **kwargs: keyword arguments for filtering materials. Fields that do not start with underscores are
+        filters, while those that start with underscores are fields to retrieve. Possible filters include:
+          - _fields (optional): list of fields to retrieve for each material
+          - Other parameters: filter criteria, where each parameter key corresponds to the field to filter and the
+            parameter value corresponds to the filter value
+
+        Returns:
+        - list of dictionaries, each dictionary representing a material retrieved based on the filtering criteria
+        """
+
+        def comma_cat(val):
+            return ",".join(val) if isinstance(val, list) else val
+
+        criteria = {k: comma_cat(v) for k, v in kwargs.items() if not k.startswith("_")}
+        params = [f"{k}={comma_cat(v)}" for k, v in kwargs.items() if k.startswith("_") and k != "_fields"]
         if "_fields" not in kwargs:
             params.append("_all_fields=True")
         else:
-            fields = ",".join(kwargs["_fields"]) if isinstance(kwargs["_fields"], list) else kwargs["_fields"]
+            fields = comma_cat(kwargs["_fields"])
             params.extend((f"_fields={fields}", "_all_fields=False"))
         get = "&".join(params)
         logger.info(f"query={get}")
-        return self.request(f"materials/summary/?{get}", payload=criteria)
+        return self.request(f"materials/{doc}/?{get}", payload=criteria)
+
+    def summary_search(self, **kwargs) -> list[dict]:
+        """
+        Mirrors mp-api's mpr.materials.summary.search functionality. Searches for materials based on the specified
+        criteria. A notable difference with the mp-api's implementation is that this uses the web API to do searches.
+        So the keywords follow the actual API spec, which is as it should be. For instance, number of sites is `nsites`
+        and number of elements is `nelements`. The mp-api package has this weird renaming that maps `num_elements` to
+        `nelements` and `num_sites` to  `nsites`.
+
+        Parameters:
+        - **kwargs: keyword arguments for filtering materials. Fields that do not start with underscores are
+        filters, while those that start with underscores are fields to retrieve. Possible filters include:
+          - _fields (optional): list of fields to retrieve for each material
+          - Other parameters: filter criteria, where each parameter key corresponds to the field to filter and the
+            parameter value corresponds to the filter value
+
+        Returns:
+        - list of dictionaries, each dictionary representing a material retrieved based on the filtering criteria
+
+        """
+        return self.search("summary", **kwargs)
 
     def get_summary(self, criteria: dict, fields: list | None = None) -> list[dict]:
         """Get  a data corresponding to a criteria.
@@ -258,14 +346,14 @@ class _MPResterBasic:
 
     def get_entries(
         self,
-        criteria,
-        compatible_only=True,
-        inc_structure=None,
-        property_data=None,
-        conventional_unit_cell=False,
-        sort_by_e_above_hull=False,
+        criteria: str | Sequence[str],
+        *,
+        compatible_only: bool = True,
+        property_data: Sequence[str] | None = None,
+        summary_data: Sequence[str] | None = None,
+        **kwargs,
     ):
-        """Get  a list of ComputedEntries or ComputedStructureEntries corresponding
+        """Get a list of ComputedEntries or ComputedStructureEntries corresponding
         to a chemical system, formula, or materials_id or full criteria.
 
         Args:
@@ -276,22 +364,41 @@ class _MPResterBasic:
                 which performs adjustments to allow mixing of GGA and GGA+U
                 calculations for more accurate phase diagrams and reaction
                 energies.
-            inc_structure (str): If None, entries returned are
-                ComputedEntries. If inc_structure="initial",
-                ComputedStructureEntries with initial structures are returned.
-                Otherwise, ComputedStructureEntries with final structures
-                are returned.
-            property_data (list): Specify additional properties to include in
-                entry.data. If None, no data. Should be a subset of
-                supported_properties.
-            conventional_unit_cell (bool): Whether to get the standard
-                conventional unit cell
-            sort_by_e_above_hull (bool): Whether to sort the list of entries by
-                e_above_hull (will query e_above_hull as a property_data if True).
+            property_data (list): Specify additional properties to include in entry.data from /materials/thermo
+                endpoint of the API.
+            summary_data (list): Specify additional properties to include in entry.data from /materials/summary
+                endpoint of the API. Note that unlike property_data, summary_data refers to the "best" calculation done
+                by MP. There are no guarantees that the summary_data will be consistent with the entry or property
+                data since the data can come from say a r2SCAN calculation but the entry is from a GGA calculation.
+                The data will be reported in the entry.data["summary"] dictionary.
+            **kwargs: Used to catch deprecated kwargs.
 
         Returns:
             List of ComputedStructureEntry objects.
         """
+        if set(kwargs.keys()).intersection({"inc_structure", "conventional_unit_cell", "sort_by_e_above_hull"}):
+            warnings.warn(
+                "The inc_structure, conventional_unit_cell, and sort_by_e_above_hull arguments are deprecated. "
+                "These arguments have no effect and will be removed in 2026.1.1.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        def proc_crit(val):
+            if val.startswith("mp-"):
+                return val
+            if "-" in val:
+                return "-".join(sorted(val.split("-")))
+            comp = Composition(val)
+            if len(comp) == 1:
+                return val
+            return comp.reduced_formula
+
+        if isinstance(criteria, str):
+            criteria = ",".join([proc_crit(c) for c in criteria.split(",")])
+        else:
+            criteria = ",".join([proc_crit(c) for c in criteria])
+
         if criteria.startswith("mp-"):
             query = f"material_ids={criteria}"
         elif "-" in criteria:
@@ -300,9 +407,14 @@ class _MPResterBasic:
             query = f"formula={criteria}"
 
         entries = []
-        response = self.request(f"materials/thermo/?_fields=entries&{query}")
+        fields = ["entries", *property_data] if property_data is not None else ["entries"]
+        response = self.request(f"materials/thermo/?_fields={','.join(fields)}&{query}")
         for dct in response:
-            entries.extend(dct["entries"].values())
+            for e in dct["entries"].values():
+                if property_data:
+                    for prop in property_data:
+                        e.data[prop] = dct[prop]
+                entries.append(e)
 
         if compatible_only:
             from pymatgen.entries.compatibility import MaterialsProject2020Compatibility
@@ -311,6 +423,20 @@ class _MPResterBasic:
             with warnings.catch_warnings():
                 warnings.filterwarnings("ignore", message="Failed to guess oxidation states.*")
                 entries = MaterialsProject2020Compatibility().process_entries(entries, clean=True)
+
+        if summary_data and len(entries) > 0:
+            for i in range(0, len(entries), CHUNK_SIZE):
+                chunked_entries = entries[i : i + CHUNK_SIZE]
+                mids = [e.data["material_id"] for e in chunked_entries]
+                edata = self.search(
+                    "summary",
+                    material_ids=mids,
+                    _fields=[*summary_data, "material_id"],
+                )
+                mapped_data = {d["material_id"]: {k: v for k, v in d.items() if k != "material_id"} for d in edata}
+                for e in chunked_entries:
+                    e.data["summary"] = mapped_data[e.data["material_id"]]
+
         return list(set(entries))
 
     def get_entry_by_material_id(self, material_id: str, *args, **kwargs) -> ComputedStructureEntry:
@@ -327,7 +453,7 @@ class _MPResterBasic:
         """
         return self.get_entries(material_id, *args, **kwargs)[0]
 
-    def get_entries_in_chemsys(self, elements, *args, **kwargs):
+    def get_entries_in_chemsys(self, elements: str | list[str], *args, **kwargs):
         """
         Helper method to get a list of ComputedEntries in a chemical system. For example, elements = ["Li", "Fe", "O"]
         will return a list of all entries in the Li-Fe-O chemical system, i.e., all LixOy, FexOy, LixFey, LixFeyOz,
@@ -343,87 +469,86 @@ class _MPResterBasic:
         Returns:
             List of ComputedEntries.
         """
-        chemsys = []
-        for i in range(1, len(elements) + 1):
-            for els in itertools.combinations(elements, i):
-                chemsys.append("-".join(sorted(els)))
+        if isinstance(elements, str):
+            elements = elements.split("-")
+        chemsys = [
+            "-".join(sorted(els)) for i in range(1, len(elements) + 1) for els in itertools.combinations(elements, i)
+        ]
         criteria = ",".join(chemsys)
 
         return self.get_entries(criteria, *args, **kwargs)
 
-    def get_phonon_bandstructure_by_material_id(self, material_id: str):
+    def _retrieve_object_from_s3(self, material_id: str, bucket: str, prefix: str, timeout: float = 60) -> Any:
+        """
+        Retrieve data from Amazon S3 OpenData the non-canonical way, using requests.
+
+        This should be transitioned to boto3 if long-term support is desired,
+        or to expand pymatgen support of, e.g., electronic DOS, bandstructure, etc.
+
+        Args:
+            material_id (str): Materials Project material_id
+            bucket (str): the Materials Project bucket, either materialsproject-parsed
+                or materialsproject-build
+            prefix (str) : the prefix of the particular S3 key.
+            timeout (float = 60) : timeout in seconds for the requests command.
+
+        Returns:
+            json loaded object
+        """
+        response = requests.get(
+            f"https://s3.us-east-1.amazonaws.com/{bucket}/{prefix}/{material_id}.json.gz",
+            timeout=timeout,
+        )
+        if response.status_code not in {200, 400}:
+            raise MPRestError(
+                f"Failed to retrieve data from OpenData with status code {response.status_code}:\n{response.reason}"
+            )
+        return orjson.loads(gzip.decompress(response.content))
+
+    def get_phonon_bandstructure_by_material_id(self, material_id: str) -> PhononBandStructureSymmLine:
         """Get phonon bandstructure by material_id.
 
+        Note that this method borrows constructor methods built into
+        in the emmet-core model for this data. Calling the `to_pmg`
+        method of the emmet-core data model handles this.
+
         Args:
             material_id (str): Materials Project material_id
 
         Returns:
             PhononBandStructureSymmLine: A phonon band structure.
         """
-        prop = "ph_bs"
-        response = self.request(f"materials/phonon/?material_ids={material_id}&_fields={prop}")
-        return response[0][prop]
+        data = self._retrieve_object_from_s3(
+            material_id, bucket="materialsproject-parsed", prefix="ph-bandstructures/dfpt"
+        )
+        rlatt = Lattice(data["reciprocal_lattice"])
+        return PhononBandStructureSymmLine(
+            [Kpoint(q, lattice=rlatt).frac_coords for q in data["qpoints"]],
+            np.array(data["frequencies"]),
+            rlatt,
+            has_nac=data["has_nac"],
+            eigendisplacements=np.array(data["eigendisplacements"]),
+            structure=data["structure"],
+            labels_dict={k: Kpoint(v, lattice=rlatt).frac_coords for k, v in (data["labels_dict"] or {}).items()},
+            coords_are_cartesian=False,
+        )
 
-    def get_phonon_dos_by_material_id(self, material_id: str):
+    def get_phonon_dos_by_material_id(self, material_id: str) -> CompletePhononDos:
         """Get phonon density of states by material_id.
 
+        Note that this method borrows constructor methods built into
+        in the emmet-core model for this data. Calling the `to_pmg`
+        method of the emmet-core data model handles this.
+
         Args:
             material_id (str): Materials Project material_id
 
         Returns:
             CompletePhononDos: A phonon DOS object.
         """
-        prop = "ph_dos"
-        response = self.request(f"materials/phonon/?material_ids={material_id}&_fields={prop}")
-        return response[0][prop]
-
-
-class MPRester:
-    """A class to conveniently interface with the new and legacy Materials Project REST interface.
-
-    The recommended way to use MPRester is as a context manager to ensure
-    that sessions are properly closed after usage:
-
-        with MPRester("API_KEY") as mpr:
-            docs = mpr.call_some_method()
-
-    MPRester uses the "requests" package, which provides HTTP connection
-    pooling. All connections are made via https for security.
-
-    For more advanced uses of the Materials API, please consult the API
-    documentation at https://materialsproject.org/api and https://docs.materialsproject.org.
-
-    This class handles the transition between old and new MP API, making it easy to switch between them
-    by passing a new (length 32) or old (15 <= length <= 17) API key. See https://docs.materialsproject.org
-    for which API to use.
-    """
-
-    def __new__(cls, *args, **kwargs) -> _MPResterNew | _MPResterBasic | _MPResterLegacy:  # type: ignore[misc]
-        """
-        Args:
-           *args: Pass through to either legacy or new MPRester.
-           **kwargs: Pass through to either legacy or new MPRester.
-        """
-        api_key = args[0] if len(args) > 0 else None
-
-        if api_key is None:
-            api_key = kwargs.get("api_key", SETTINGS.get("PMG_MAPI_KEY"))
-            kwargs["api_key"] = api_key
-
-        if not api_key:
-            raise ValueError("Please supply an API key. See https://materialsproject.org/api for details.")
-
-        if len(api_key) != 32:
-            from pymatgen.ext.matproj_legacy import _MPResterLegacy
-
-            return _MPResterLegacy(*args, **kwargs)
-
-        try:
-            from mp_api.client import MPRester as _MPResterNew
-
-            return _MPResterNew(*args, **kwargs)
-        except Exception:
-            return _MPResterBasic(*args, **kwargs)
+        data = self._retrieve_object_from_s3(material_id, bucket="materialsproject-parsed", prefix="ph-dos/dfpt")
+        data["pdos"] = data.pop("projected_densities", None)
+        return CompletePhononDos.from_dict(data)
 
 
 class MPRestError(Exception):
