@@ -30,12 +30,14 @@ from pymatgen.symmetry.structure import SymmetrizedStructure
 from pymatgen.util.coord import find_in_coord_list_pbc, in_coord_list_pbc
 
 if TYPE_CHECKING:
+    from collections.abc import Sequence
     from typing import Any
 
     from numpy.typing import NDArray
     from typing_extensions import Self
 
-    from pymatgen.util.typing import PathLike, Vector3D
+    from pymatgen.core import IStructure
+    from pymatgen.util.typing import MagMomentLike, PathLike
 
 __author__ = "Shyue Ping Ong, Will Richards, Matthew Horton"
 
@@ -300,7 +302,7 @@ class CifFile:
             CifFile
         """
         with zopen(filename, mode="rt", errors="replace", encoding="utf-8") as file:
-            return cls.from_str(file.read())
+            return cls.from_str(file.read())  # type:ignore[arg-type]
 
 
 class CifParser:
@@ -598,10 +600,10 @@ class CifParser:
 
     def _unique_coords(
         self,
-        coords: list[Vector3D],
-        magmoms: list[Magmom] | None = None,
+        coords: list[tuple[float, float, float]],
+        magmoms: Sequence[MagMomentLike] | None = None,
         lattice: Lattice | None = None,
-        labels: dict[Vector3D, str] | None = None,
+        labels: dict[tuple[float, float, float], str] | None = None,
     ) -> tuple[list[NDArray], list[Magmom], list[str]]:
         """Generate unique coordinates using coordinates and symmetry
         positions, and their corresponding magnetic moments if supplied.
@@ -858,7 +860,7 @@ class CifParser:
                             MagSymmOp.from_rotation_and_translation_and_time_reversal(
                                 rotation_matrix=op.rotation_matrix,
                                 translation_vec=new_translation,
-                                time_reversal=cast(Literal[-1, 1], new_time_reversal),
+                                time_reversal=cast("Literal[-1, 1]", new_time_reversal),
                             )
                         )
                 mag_symm_ops = all_ops
@@ -992,11 +994,11 @@ class CifParser:
             return num_h.get(symbol[:3], 0)
 
         def get_matching_coord(
-            coord_to_species: dict[Vector3D, Composition],
-            coord: Vector3D,
-        ) -> Vector3D | Literal[False]:
+            coord_to_species: dict[tuple[float, float, float], Composition],
+            coord: tuple[float, float, float],
+        ) -> tuple[float, float, float] | Literal[False]:
             """Find site by coordinate."""
-            coords: list[Vector3D] = list(coord_to_species)
+            coords: list[tuple[float, float, float]] = list(coord_to_species)
             for op in self.symmetry_operations:
                 frac_coord = op.operate(coord)
                 indices: NDArray = find_in_coord_list_pbc(coords, frac_coord, atol=self._site_tolerance)
@@ -1024,14 +1026,14 @@ class CifParser:
             magmoms = self._parse_magmoms(data)
 
         else:
-            self.symmetry_operations = self.get_symops(data)
+            self.symmetry_operations = self.get_symops(data)  # type:ignore[assignment]
             magmoms = {}
 
         oxi_states = self._parse_oxi_states(data)
 
-        coord_to_species: dict[Vector3D, Composition] = {}
-        coord_to_magmoms: dict[Vector3D, NDArray] = {}
-        labels: dict[Vector3D, str] = {}
+        coord_to_species: dict[tuple[float, float, float], Composition] = {}
+        coord_to_magmoms: dict[tuple[float, float, float], NDArray] = {}
+        labels: dict[tuple[float, float, float], str] = {}
 
         for idx, label in enumerate(data["_atom_site_label"]):
             # If site type symbol exists, use it. Otherwise use the label
@@ -1068,7 +1070,7 @@ class CifParser:
             # If don't check_occu or the occupancy is greater than 0, create comp_dict
             if not check_occu or occu > 0:
                 # Create site coordinate
-                coord: Vector3D = (
+                coord: tuple[float, float, float] = (
                     str2float(data["_atom_site_fract_x"][idx]),
                     str2float(data["_atom_site_fract_y"][idx]),
                     str2float(data["_atom_site_fract_z"][idx]),
@@ -1086,8 +1088,8 @@ class CifParser:
                 comp = Composition(comp_dict)
 
                 # Find matching site by coordinate
-                match: Vector3D | Literal[False] = get_matching_coord(coord_to_species, coord)
-                if not match:
+                match: tuple[float, float, float] | Literal[False] = get_matching_coord(coord_to_species, coord)
+                if match is False:
                     coord_to_species[coord] = comp
                     coord_to_magmoms[coord] = magmoms.get(label, np.array([0, 0, 0]))
                     labels[coord] = label
@@ -1095,7 +1097,7 @@ class CifParser:
                 else:
                     coord_to_species[match] += comp
                     # Disordered magnetic currently not supported
-                    coord_to_magmoms[match] = None
+                    coord_to_magmoms[match] = None  # type:ignore[assignment]
                     labels[match] = label
 
         # Check occupancy
@@ -1117,7 +1119,7 @@ class CifParser:
         # Collect info for building Structure
         all_species: list[Composition] = []
         all_species_noedit: list[Composition] = []
-        all_coords: list[Vector3D] = []
+        all_coords: list[tuple[float, float, float]] = []
         all_magmoms: list[Magmom] = []
         all_hydrogens: list[float] = []
         equivalent_indices: list[int] = []
@@ -1141,8 +1143,8 @@ class CifParser:
                     key=lambda x: x[1],
                 )
             ):
-                tmp_coords: list[Vector3D] = [site[0] for site in group]
-                tmp_magmom: list[Magmom] = [coord_to_magmoms[tmp_coord] for tmp_coord in tmp_coords]
+                tmp_coords: list[tuple[float, float, float]] = [site[0] for site in group]
+                tmp_magmom: list[NDArray] = [coord_to_magmoms[tmp_coord] for tmp_coord in tmp_coords]
 
                 if self.feature_flags["magcif"]:
                     coords, _magmoms, new_labels = self._unique_coords(
@@ -1174,7 +1176,7 @@ class CifParser:
                 equivalent_indices += len(coords) * [idx]
 
                 all_hydrogens.extend(len(coords) * [im_h])
-                all_coords.extend(coords)
+                all_coords.extend(coords)  # type:ignore[arg-type]
                 all_species.extend(len(coords) * [species])
                 all_magmoms.extend(_magmoms)
                 all_labels.extend(new_labels)
@@ -1209,7 +1211,7 @@ class CifParser:
                 all_labels = None  # type: ignore[assignment]
 
             struct: Structure = Structure(
-                lattice,
+                lattice,  # type:ignore[arg-type]
                 all_species,
                 all_coords,
                 site_properties=site_properties,
@@ -1349,16 +1351,15 @@ class CifParser:
     def get_bibtex_string(self) -> str:
         """Get BibTeX reference from CIF file.
 
-        Args:
-            data:
+        TODO:
+            - parse '_publ_section_references' when it exists?
+            - CIF specification supports multiple citations.
 
         Returns:
             BibTeX string.
         """
-        try:
-            from pybtex.database import BibliographyData, Entry
-        except ImportError:
-            raise RuntimeError("Bibliographic data extraction requires pybtex.")
+        from bibtexparser.bibdatabase import BibDatabase
+        from bibtexparser.bwriter import BibTexWriter
 
         bibtex_keys: dict[str, tuple[str, ...]] = {
             "author": ("_publ_author_name", "_citation_author_name"),
@@ -1377,44 +1378,45 @@ class CifParser:
             "doi": ("_journal_DOI", "_citation_DOI"),
         }
 
-        entries: dict[str, Entry] = {}
-
-        # TODO: parse '_publ_section_references' when it exists?
-        # TODO: CIF specification supports multiple citations.
+        db = BibDatabase()
+        db.entries = []
 
         for idx, data in enumerate(self._cif.data.values()):
             # Convert to lower-case keys, some CIF files inconsistent
             _data = {k.lower(): v for k, v in data.data.items()}
-
-            bibtex_entry = {}
+            entry = {"ENTRYTYPE": "article", "ID": f"cifref{idx}"}
 
             for field, tags in bibtex_keys.items():
                 for tag in tags:
                     if tag in _data:
-                        if isinstance(_data[tag], list):
-                            bibtex_entry[field] = _data[tag][0]
-                        else:
-                            bibtex_entry[field] = _data[tag]
+                        value = _data[tag]
+                        entry[field] = value[0] if isinstance(value, list) else value
+                        break
 
             # Convert to bibtex author format ("and" delimited)
-            if "author" in bibtex_entry:
+            if "author" in entry:
                 # Separate out semicolon authors
-                if isinstance(bibtex_entry["author"], str) and ";" in bibtex_entry["author"]:
-                    bibtex_entry["author"] = bibtex_entry["author"].split(";")
-
-                if isinstance(bibtex_entry["author"], list):
-                    bibtex_entry["author"] = " and ".join(bibtex_entry["author"])
+                if isinstance(entry["author"], str) and ";" in entry["author"]:
+                    entry["author"] = entry["author"].split(";")  # type:ignore[assignment]
+                if isinstance(entry["author"], list):
+                    entry["author"] = " and ".join(entry["author"])
 
             # Convert to bibtex page range format, use empty string if not specified
-            if ("page_first" in bibtex_entry) or ("page_last" in bibtex_entry):
-                bibtex_entry["pages"] = bibtex_entry.get("page_first", "") + "--" + bibtex_entry.get("page_last", "")
-                bibtex_entry.pop("page_first", None)  # and remove page_first, page_list if present
-                bibtex_entry.pop("page_last", None)
+            if "page_first" in entry or "page_last" in entry:
+                entry["pages"] = f"{entry.get('page_first', '')}--{entry.get('page_last', '')}"
+                entry.pop("page_first", None)  # and remove page_first, page_list if present
+                entry.pop("page_last", None)
 
-            # Cite keys are given as cif-reference-idx in order they are found
-            entries[f"cifref{idx}"] = Entry("article", list(bibtex_entry.items()))
+            db.entries.append(entry)
 
-        return BibliographyData(entries).to_string(bib_format="bibtex")
+        # NOTE: the following is added to make output consistent with
+        # previous pybtex implementation
+        writer = BibTexWriter()
+        writer.indent = "    "
+        writer.display_order = ("author", "title", "journal", "volume", "year", "pages")
+
+        # Replace curly brackets with double quotes (skip the first and last one)
+        return re.sub(r"(^\s*\w+\s*=\s*)\{([^{}]*)\}", r'\1"\2"', writer.write(db), flags=re.MULTILINE)
 
     def as_dict(self) -> dict:
         """MSONable dict."""
@@ -1533,7 +1535,7 @@ class CifWriter:
 
     def __init__(
         self,
-        struct: Structure,
+        struct: Structure | IStructure,
         symprec: float | None = None,
         write_magmoms: bool = False,
         significant_figures: int = 8,
@@ -1765,4 +1767,4 @@ class CifWriter:
     ) -> None:
         """Write the CIF file."""
         with zopen(filename, mode=mode, encoding="utf-8") as file:
-            file.write(str(self))
+            file.write(str(self))  # type:ignore[arg-type]
