@@ -5,6 +5,7 @@ for parsing CP2K-related outputs.
 
 from __future__ import annotations
 
+import logging
 import os
 import re
 import warnings
@@ -32,6 +33,9 @@ __version__ = "2.0"
 __status__ = "Production"
 
 
+logger = logging.getLogger(__name__)
+
+
 class Cp2kOutput:
     """Parse output file from CP2K. The CP2K output file is very flexible in the way that
     it is returned. This class will automatically parse parameters that should always be present,
@@ -946,7 +950,7 @@ class Cp2kOutput:
             footer_pattern=footer,
             last_one_only=False,
         ):
-            print("Found data, but not yet implemented!")
+            logger.warning("Found data, but not yet implemented!")
 
     def parse_hirshfeld(self):
         """Parse the Hirshfeld population analysis for each step."""
