@@ -71,7 +71,8 @@ class LDos(MSONable):
             begin = 0
 
             with zopen(pot_inp, mode="rt", encoding="utf-8") as potfile:
-                for line in potfile:
+                line: str
+                for line in potfile:  # type:ignore[assignment]
                     if len(pot_read_end.findall(line)) > 0:
                         break
 
@@ -99,7 +100,6 @@ class LDos(MSONable):
             lines = file.readlines()
         e_fermi = float(lines[0].split()[4])
 
-        dos_energies = []
         ldos = {}
 
         for idx in range(1, len(pot_dict) + 1):
@@ -108,8 +108,7 @@ class LDos(MSONable):
             else:
                 ldos[idx] = np.loadtxt(f"{ldos_file}{idx}.dat")
 
-        for idx in range(len(ldos[1])):
-            dos_energies.append(ldos[1][idx][0])
+        dos_energies = [ldos[1][idx][0] for idx in range(len(ldos[1]))]
 
         all_pdos: list[dict] = []
         vorb = {"s": Orbital.s, "p": Orbital.py, "d": Orbital.dxy, "f": Orbital.f0}
@@ -144,7 +143,7 @@ class LDos(MSONable):
         _t_dos: dict = {Spin.up: t_dos}
 
         dos = Dos(e_fermi, dos_energies, _t_dos)
-        complete_dos = CompleteDos(structure, dos, pdoss)
+        complete_dos = CompleteDos(structure, dos, pdoss)  # type:ignore[arg-type]
         charge_transfer = LDos.charge_transfer_from_file(feff_inp_file, ldos_file)
         return cls(complete_dos, charge_transfer)
 
