@@ -86,7 +86,7 @@ class GaussianInput:
     """A Gaussian input file."""
 
     # Commonly used regex patterns
-    _zmat_patt = re.compile(r"^(\w+)*([\s,]+(\w+)[\s,]+(\w+))*[\-\.\s,\w]*$")
+    _zmat_patt = re.compile(r"^(\w+)([\s,]+(\w+)[\s,]+(\w+)){0,3}[\-\.\s,\w]*$")
     _xyz_patt = re.compile(r"^(\w+)[\s,]+([\d\.eE\-]+)[\s,]+([\d\.eE\-]+)[\s,]+([\d\.eE\-]+)[\-\.\s,\w.]*$")
 
     def __init__(
@@ -369,7 +369,7 @@ class GaussianInput:
             GaussianInput object
         """
         with zopen(filename, mode="rt", encoding="utf-8") as file:
-            return cls.from_str(file.read())
+            return cls.from_str(file.read())  # type:ignore[arg-type]
 
     def get_zmatrix(self):
         """Get a z-matrix representation of the molecule."""
@@ -377,9 +377,7 @@ class GaussianInput:
 
     def get_cart_coords(self) -> str:
         """Return the Cartesian coordinates of the molecule."""
-        outs = []
-        for site in self._mol:
-            outs.append(f"{site.species_string} {' '.join(f'{x:0.6f}' for x in site.coords)}")
+        outs = [f"{site.species_string} {' '.join(f'{x:0.6f}' for x in site.coords)}" for site in self._mol]
         return "\n".join(outs)
 
     def __str__(self):
@@ -820,17 +818,17 @@ class GaussianOutput:
                     elif parse_freq:
                         while line.strip() != "":  # blank line
                             ifreqs = [int(val) - 1 for val in line.split()]
-                            for _ in ifreqs:
-                                frequencies.append(
-                                    {
-                                        "frequency": None,
-                                        "r_mass": None,
-                                        "f_constant": None,
-                                        "IR_intensity": None,
-                                        "symmetry": None,
-                                        "mode": [],
-                                    }
-                                )
+                            frequencies.extend(
+                                {
+                                    "frequency": None,
+                                    "r_mass": None,
+                                    "f_constant": None,
+                                    "IR_intensity": None,
+                                    "symmetry": None,
+                                    "mode": [],
+                                }
+                                for _ in ifreqs
+                            )
                             # read freq, intensity, masses, symmetry ...
                             while "Atom  AN" not in line:
                                 if "Frequencies --" in line:
