@@ -8,10 +8,15 @@ from __future__ import annotations
 import pprint
 import warnings
 from dataclasses import dataclass, field
-from typing import Any
+from typing import TYPE_CHECKING, Any
+
+if TYPE_CHECKING:
+    import numpy as np
+
+from monty.dev import deprecated
 
 from pymatgen.core.units import Ha_to_eV
-from pymatgen.io.jdftx._output_utils import get_colon_var_t1
+from pymatgen.io.jdftx._output_utils import get_colon_val
 
 __author__ = "Ben Rich"
 
@@ -46,15 +51,15 @@ class JElStep:
     etype: str | None = None
     nstep: int | None = None
     e: float | None = None
-    grad_k: float | None = None
-    alpha: float | None = None
-    linmin: float | None = None
-    t_s: float | None = None
-    mu: float | None = None
-    nelectrons: float | None = None
-    abs_magneticmoment: float | None = None
-    tot_magneticmoment: float | None = None
-    subspacerotationadjust: float | None = None
+    grad_k: float | np.float64 | None = None
+    alpha: float | np.float64 | None = None
+    linmin: float | np.float64 | None = None
+    t_s: float | np.float64 | None = None
+    mu: float | np.float64 | None = None
+    nelectrons: float | np.float64 | None = None
+    abs_magneticmoment: float | np.float64 | None = None
+    tot_magneticmoment: float | np.float64 | None = None
+    subspacerotationadjust: float | np.float64 | None = None
     converged: bool = False
     converged_reason: str | None = None
 
@@ -111,16 +116,16 @@ class JElStep:
         Args:
             line_text (str): A line of text from a JDFTx out file containing the electronic minimization data.
         """
-        nstep_float = get_colon_var_t1(line_text, "Iter: ")
+        nstep_float = get_colon_val(line_text, "Iter: ")
         if isinstance(nstep_float, float):
             self.nstep = int(nstep_float)
         elif nstep_float is None:
             raise ValueError("Could not find nstep in line_text")
-        self.e = get_colon_var_t1(line_text, f"{self.etype}: ") * Ha_to_eV
-        self.grad_k = get_colon_var_t1(line_text, "|grad|_K: ")
-        self.alpha = get_colon_var_t1(line_text, "alpha: ")
-        self.linmin = get_colon_var_t1(line_text, "linmin: ")
-        self.t_s = get_colon_var_t1(line_text, "t[s]: ")
+        self.e = get_colon_val(line_text, f"{self.etype}: ") * Ha_to_eV
+        self.grad_k = get_colon_val(line_text, "|grad|_K: ")
+        self.alpha = get_colon_val(line_text, "alpha: ")
+        self.linmin = get_colon_val(line_text, "linmin: ")
+        self.t_s = get_colon_val(line_text, "t[s]: ")
 
     def _is_fillings_line(self, i: int, line_text: str) -> bool:
         """Return True if fillings line.
@@ -177,7 +182,7 @@ class JElStep:
             line_text (str): A line of text from a JDFTx out file containing the electronic
             minimization data.
         """
-        self.subspacerotationadjust = get_colon_var_t1(line_text, "SubspaceRotationAdjust: set factor to")
+        self.subspacerotationadjust = get_colon_val(line_text, "SubspaceRotationAdjust: set factor to")
 
     def _set_magdata(self, fillings_line: str) -> None:
         """Set class variables abs_magneticMoment, tot_magneticMoment.
@@ -190,8 +195,8 @@ class JElStep:
             minimization data.
         """
         _fillings_line = fillings_line.split("magneticMoment: [ ")[1].split(" ]")[0].strip()
-        self.abs_magneticmoment = get_colon_var_t1(_fillings_line, "Abs: ")
-        self.tot_magneticmoment = get_colon_var_t1(_fillings_line, "Tot: ")
+        self.abs_magneticmoment = get_colon_val(_fillings_line, "Abs: ")
+        self.tot_magneticmoment = get_colon_val(_fillings_line, "Tot: ")
 
     def _set_mu(self, fillings_line: str) -> None:
         """Set mu class variable.
@@ -203,7 +208,7 @@ class JElStep:
             fillings_line (str): A line of text from a JDFTx out file containing the electronic
             minimization data.
         """
-        self.mu = get_colon_var_t1(fillings_line, "mu: ") * Ha_to_eV
+        self.mu = get_colon_val(fillings_line, "mu: ") * Ha_to_eV
 
     def _set_nelectrons(self, fillings_line: str) -> None:
         """Set nelectrons class variable.
@@ -214,9 +219,9 @@ class JElStep:
         Args:
             fillings_line(str): A line of text from a JDFTx out file containing the electronic minimization data
         """
-        self.nelectrons = get_colon_var_t1(fillings_line, "nElectrons: ")
+        self.nelectrons = get_colon_val(fillings_line, "nElectrons: ")
 
-    def to_dict(self) -> dict:
+    def as_dict(self) -> dict:
         """Return dictionary representation of JElStep object.
 
         Returns:
@@ -225,12 +230,16 @@ class JElStep:
         dct = {}
         for fld in self.__dataclass_fields__:
             value = getattr(self, fld)
-            if hasattr(value, "to_dict"):
-                dct[fld] = value.to_dict()
+            if hasattr(value, "as_dict"):
+                dct[fld] = value.as_dict()
             else:
                 dct[fld] = value
         return dct
 
+    @deprecated(as_dict, deadline=(2025, 10, 4))
+    def to_dict(self):
+        return self.as_dict()
+
     def __str__(self) -> str:
         """
         Return string representation of JElStep object.
@@ -338,11 +347,10 @@ class JElSteps:
             etype (str): The type of energy component.
         """
         line_collections, lines_collect = _gather_JElSteps_line_collections(opt_type, text_slice)
-        slices = []
+        slices = [JElStep._from_lines_collect(_lines_collect, opt_type, etype) for _lines_collect in line_collections]
         converged = None
         converged_reason = None
-        for _lines_collect in line_collections:
-            slices.append(JElStep._from_lines_collect(_lines_collect, opt_type, etype))
+
         if len(lines_collect):
             converged, converged_reason = _parse_ending_lines(lines_collect, opt_type)
         instance = cls(slices=slices, converged=converged, converged_reason=converged_reason)
@@ -350,14 +358,41 @@ class JElSteps:
         instance.etype = etype
         return instance
 
+    @classmethod
+    def _from_nothing(cls, opt_type: str = "ElecMinimize", etype: str = "F") -> JElSteps:
+        """Return JElSteps object.
+
+        Create an empty JElSteps object.
+
+        Args:
+            opt_type (str): The type of electronic minimization step.
+            etype (str): The type of energy component.
+        """
+        slices: list[JElStep] = []
+        converged = False
+        converged_reason = None
+        instance = cls(slices=slices, converged=converged, converged_reason=converged_reason)
+        instance.opt_type = opt_type
+        instance.etype = etype
+        return instance
+
     def __post_init__(self) -> None:
         """Post initialization method."""
         if len(self.slices):
             self.nstep = self._get_nstep()
             for var in _jelsteps_atrs_from_last_slice:
-                setattr(self, var, getattr(self.slices[-1], var))
+                val = None
+                for i in range(1, len(self.slices) + 1):
+                    val = getattr(self.slices[-i], var)
+                    if val is not None:
+                        break
+                setattr(self, var, val)
+        else:
+            self.nstep = 0
+            for var in _jelsteps_atrs_from_last_slice:
+                setattr(self, var, None)
 
-    def to_dict(self) -> dict[str, Any]:
+    def as_dict(self) -> dict[str, Any]:
         """Return dictionary representation of JElSteps object.
 
         Returns:
@@ -366,15 +401,19 @@ class JElSteps:
         dct = {}
         for fld in self.__dataclass_fields__:
             if fld == "slices":
-                dct[fld] = [slc.to_dict() for slc in self.slices]
+                dct[fld] = [slc.as_dict() for slc in self.slices]
                 continue
             value = getattr(self, fld)
-            if hasattr(value, "to_dict"):
-                dct[fld] = value.to_dict()
+            if hasattr(value, "as_dict"):
+                dct[fld] = value.as_dict()
             else:
                 dct[fld] = value
         return dct
 
+    @deprecated(as_dict, deadline=(2025, 10, 4))
+    def to_dict(self):
+        return self.as_dict()
+
     def __getitem__(self, key: int | str) -> JElStep | Any:
         """Return item.
 
