@@ -14,18 +14,29 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any
 
 import numpy as np
+from monty.dev import deprecated
 
+from pymatgen.core import Lattice
+from pymatgen.core.units import Ha_to_eV
+from pymatgen.electronic_structure.bandstructure import BandStructure
+from pymatgen.electronic_structure.core import Spin
 from pymatgen.io.jdftx._output_utils import (
     _get_nbands_from_bandfile_filepath,
     _get_orb_label_list,
+    _get_u_to_oa_map,
+    _parse_kptsfrom_bandprojections_file,
     get_proj_tju_from_file,
+    orb_ref_list,
     read_outfile_slices,
 )
 from pymatgen.io.jdftx.jdftxoutfileslice import JDFTXOutfileSlice
 
 if TYPE_CHECKING:
+    from numpy.typing import NDArray
+
     from pymatgen.core.structure import Structure
     from pymatgen.core.trajectory import Trajectory
+    from pymatgen.io.jdftx.inputs import JDFTXInfile
     from pymatgen.io.jdftx.jelstep import JElSteps
     from pymatgen.io.jdftx.jminsettings import (
         JMinSettingsElectronic,
@@ -41,7 +52,7 @@ __author__ = "Ben Rich, Jacob Clary"
 # This will be a long list so keep alphabetical to make adding more files easier.
 # Dump files that are redundant to data contained in the outfile attribute are included for completeness
 # but commented out.
-dump_file_names = [
+dump_file_names = (
     "bandProjections",
     "dosUp",
     "dosDn",
@@ -61,12 +72,14 @@ dump_file_names = [
     "tau_up",
     "tau_dn",
     "wfns",
-]
+)
 
-implemented_store_vars = [
+implemented_store_vars = (
     "bandProjections",
     "eigenvals",
-]
+    "kpts",
+    "bandstructure",
+)
 
 
 @dataclass
@@ -102,6 +115,8 @@ class JDFTXOutputs:
             0-based index will appear mimicking a principle quantum number (ie "0px" for first shell and "1px" for
             second shell). The actual principal quantum number is not stored in the JDFTx output files and must be
             inferred by the user.
+        kpts (list[np.ndarray]): A list of the k-points used in the calculation. Each k-point is a 3D numpy array.
+        wk_list (list[np.ndarray]): A list of the weights for the k-points used in the calculation.
     """
 
     calc_dir: str | Path = field(init=True)
@@ -109,10 +124,13 @@ class JDFTXOutputs:
     store_vars: list[str] = field(default_factory=list, init=True)
     paths: dict[str, Path] = field(init=False)
     outfile: JDFTXOutfile = field(init=False)
-    bandProjections: np.ndarray | None = field(init=False)
-    eigenvals: np.ndarray | None = field(init=False)
+    bandProjections: NDArray | None = field(init=False)
+    eigenvals: NDArray | None = field(init=False)
+    kpts: list[NDArray] | None = field(init=False)
+    wk_list: list[NDArray] | None = field(init=False)
     # Misc metadata for interacting with the data
     orb_label_list: tuple[str, ...] | None = field(init=False)
+    bandstructure: BandStructure | None = field(init=False)
 
     @classmethod
     def from_calc_dir(
@@ -134,13 +152,22 @@ class JDFTXOutputs:
         Returns:
             JDFTXOutputs: The JDFTXOutputs object.
         """
-        if store_vars is None:
-            store_vars = []
+        store_vars = cls._check_store_vars(store_vars)
         return cls(calc_dir=Path(calc_dir), store_vars=store_vars, outfile_name=outfile_name)
 
     def __post_init__(self):
         self._init_paths()
         self._store_vars()
+        self._init_bandstructure()
+
+    @staticmethod
+    def _check_store_vars(store_vars: list[str] | None) -> list[str]:
+        if store_vars is None:
+            return []
+        if "bandstructure" in store_vars:
+            store_vars += ["kpts", "eigenvals"]
+            store_vars.pop(store_vars.index("bandstructure"))
+        return list(set(store_vars))
 
     def _init_paths(self):
         self.paths = {}
@@ -206,10 +233,105 @@ class JDFTXOutputs:
         if "eigenvals" in self.paths:
             self.eigenvals = np.fromfile(self.paths["eigenvals"])
             nstates = int(len(self.eigenvals) / self.outfile.nbands)
-            self.eigenvals = self.eigenvals.reshape(nstates, self.outfile.nbands)
+            self.eigenvals = self.eigenvals.reshape(nstates, self.outfile.nbands) * Ha_to_eV
+
+    def _check_kpts(self):
+        if "bandProjections" in self.paths and self.paths["bandProjections"].exists():
+            # TODO: Write kpt file inconsistency checking
+            return
+        if "kPts" in self.paths and self.paths["kPts"].exists():
+            raise NotImplementedError("kPts file parsing not yet implemented.")
+        raise RuntimeError("No k-point data found in JDFTx output files.")
+
+    def _store_kpts(self):
+        if "bandProjections" in self.paths and self.paths["bandProjections"].exists():
+            wk_list, kpts_list = _parse_kptsfrom_bandprojections_file(self.paths["bandProjections"])
+            self.kpts = kpts_list
+            self.wk_list = wk_list
+
+    def _init_bandstructure(self):
+        if True in [v is None for v in [self.kpts, self.eigenvals]]:
+            return
+        kpoints = np.array(self.kpts)
+        eigenvals = self._get_pmg_eigenvals()
+        projections = None
+        if self.bandProjections is not None:
+            projections = self._get_pmg_projections()
+        self.bandstructure = BandStructure(
+            kpoints,
+            eigenvals,
+            Lattice(self.outfile.structure.lattice.reciprocal_lattice.matrix),
+            self.outfile.mu,
+            projections=projections,
+            structure=self.outfile.structure,
+        )
+
+    def _get_lmax(self) -> tuple[str | None, int | None]:
+        """Get the maximum l quantum number and projection array orbital length.
+
+        Returns:
+            tuple[str | None, int | None]: The maximum l quantum number and projection array orbital length.
+                Both are None if no bandProjections file is available.
+        """
+        if self.orb_label_list is None:
+            return None, None
+        orbs = [label.split("(")[1].split(")")[0] for label in self.orb_label_list]
+        if orb_ref_list[-1][0] in orbs:
+            return "f", 16
+        if orb_ref_list[-2][0] in orbs:
+            return "d", 9
+        if orb_ref_list[-3][0] in orbs:
+            return "p", 4
+        if orb_ref_list[-4][0] in orbs:
+            return "s", 1
+        raise ValueError("Unrecognized orbital labels in orb_label_list.")
+
+    def _get_pmg_eigenvals(self) -> dict | None:
+        if self.eigenvals is None:
+            return None
+        _e_skj = self.eigenvals.copy().reshape(self.outfile.nspin, -1, self.outfile.nbands)
+        _e_sjk = np.swapaxes(_e_skj, 1, 2)
+        spins = [Spin.up, Spin.down]
+        eigenvals = {}
+        for i in range(self.outfile.nspin):
+            eigenvals[spins[i]] = _e_sjk[i]
+        return eigenvals
+
+    def _get_pmg_projections(self) -> dict | None:
+        """Return pymatgen-compatible projections dictionary.
+
+        Converts the bandProjections array to a pymatgen-compatible dictionary
+
+        Returns:
+            dict | None:
+        """
+        _lmax, norbmax = self._get_lmax()
+        if norbmax is None:
+            return None
+        if self.orb_label_list is None:
+            return None
+        _proj_tju = self.bandProjections.copy()
+        if _proj_tju.dtype is np.complex64:
+            # Convert <orb|band(state)> to |<orb|band(state)>|^2
+            _proj_tju = np.abs(_proj_tju) ** 2
+        # Convert to standard datatype - using np.real to suppress warnings
+        proj_tju = np.array(np.real(_proj_tju), dtype=float)
+        proj_skju = proj_tju.reshape([self.outfile.nspin, -1, self.outfile.nbands, len(self.orb_label_list)])
+        proj_sjku = np.swapaxes(proj_skju, 1, 2)
+        nspin, nbands, nkpt, nproj = proj_sjku.shape
+        u_to_oa_map = _get_u_to_oa_map(self.paths["bandProjections"])
+        projections = {}
+        spins = [Spin.up, Spin.down]
+        for i in range(nspin):
+            projections[spins[i]] = np.zeros([nbands, nkpt, norbmax, len(self.outfile.structure)])
+            # TODO: Consider jitting this loop
+            for u in range(nproj):
+                idx = (slice(None), slice(None), *tuple(u_to_oa_map[u]))
+                projections[spins[i]][idx] += proj_sjku[i, :, :, u]
+        return projections
 
 
-_jof_atr_from_last_slice = [
+_jof_atr_from_last_slice = (
     "prefix",
     "jstrucs",
     "jsettings_fluid",
@@ -286,7 +408,44 @@ _jof_atr_from_last_slice = [
     "electronic_output",
     "t_s",
     "ecomponents",
-]
+    "infile",
+    "vibrational_modes",
+    "vibrational_energy_components",
+)
+
+# TODO: Remove references to the deprecated 'jsettings_*' attributes in `JDFTXOutfile` and `JDFTXOutfileSlice`
+# TODO: JDFTXOutfile and JDFTXOutfileSlice are VERY bloated. The following attributes are redundant
+# to fields of the `Structure` object and their removal likely won't cause any confusion.
+# - lattice
+# - lattice_final
+# - atom_coords
+# - atom_coords_final
+# - a/b/c
+# The following attributes are redundant to the internal `JDFTXInfile` object, but since the JDFTXInfile
+# object is not a standard pymatgen object, it may be better to decorate them with a deprecated decorator
+# until end of year. Additionally, it should be made certain which of these are dumped in the outfile's
+# input section regardless of what was set in the input file - for those that are not, they need to be kept.
+# - lattice_initial
+# - atom_coords_initial
+# - broadening
+# - broadening_type
+# - kgrid
+# - truncation_type
+# - truncation_radius
+# - fluid
+# - pwcut
+# - rhocut
+# The following are attributes that come from the electronic/fluid/ionic/lattice optimization step
+# logs. I am not sure if it is actually helpful to have access to these from the `JDFTXOutfile` object,
+# as they are only informative for analyzing optimization convergence. Additionally, the fields are
+# less universal than I thought, and can change depending on the optimization settings, so it might
+# be smarter to store them in a dictionary of arbitrary keys instead within the contained substructures.
+# - grad_k
+# - alpha
+# - linmin
+# - elec_grad_k
+# - elec_alpha
+# - elec_linmin
 
 
 @dataclass
@@ -419,6 +578,7 @@ class JDFTXOutfile:
         elec_alpha (float): The step size of the final electronic step in the most recent JDFTx call.
         elec_linmin (float): The final normalized projection of the electronic step direction onto the gradient for the
             most recent JDFTx call.
+        infile (JDFTXInfile): The JDFTXInfile object representing the input parameters of the JDFTXOutfile.
 
     Magic Methods:
         __getitem__(key: str | int) -> Any: Decides behavior of how JDFTXOutfile objects are indexed. If the key is a
@@ -429,7 +589,7 @@ class JDFTXOutfile:
         __str__() -> str: Returns a string representation of the JDFTXOutfile object.
     """
 
-    slices: list[JDFTXOutfileSlice] = field(default_factory=list)
+    slices: list[JDFTXOutfileSlice | None] = field(default_factory=list)
     prefix: str = field(init=False)
     jstrucs: JOutStructures = field(init=False)
     jsettings_fluid: JMinSettingsFluid = field(init=False)
@@ -505,10 +665,13 @@ class JDFTXOutfile:
     elec_alpha: float = field(init=False)
     elec_linmin: float = field(init=False)
     electronic_output: float = field(init=False)
+    infile: JDFTXInfile = field(init=False)
+    vibrational_modes: list[dict[str, Any]] | None = None
+    vibrational_energy_components: dict[str, float] | None = None
 
     @classmethod
     def from_calc_dir(
-        cls, calc_dir: str | Path, is_bgw: bool = False, none_slice_on_error: bool = False
+        cls, calc_dir: str | Path, is_bgw: bool = False, none_slice_on_error: bool | None = None
     ) -> JDFTXOutfile:
         """
         Create a JDFTXOutfile object from a directory containing JDFTx out files.
@@ -525,11 +688,7 @@ class JDFTXOutfile:
             JDFTXOutfile: The JDFTXOutfile object.
         """
         file_path = _find_jdftx_out_file(Path(calc_dir))
-        texts = read_outfile_slices(file_path)
-        slices = [
-            JDFTXOutfileSlice._from_out_slice(text, is_bgw=is_bgw, none_on_error=none_slice_on_error) for text in texts
-        ]
-        return cls(slices=slices)
+        return cls.from_file(file_path=file_path, is_bgw=is_bgw, none_slice_on_error=none_slice_on_error)
 
     @classmethod
     def from_file(
@@ -550,7 +709,7 @@ class JDFTXOutfile:
         Returns:
             JDFTXOutfile: The JDFTXOutfile object.
         """
-        texts = read_outfile_slices(file_path)
+        texts = read_outfile_slices(str(file_path))
         if none_slice_on_error is None:
             none_slice_bools = [i != len(texts) - 1 for i in range(len(texts))]
         else:
@@ -561,10 +720,26 @@ class JDFTXOutfile:
         ]
         return cls(slices=slices)
 
+    # TODO: Write testing for this function
+    def to_jdftxinfile(self) -> JDFTXInfile:
+        """
+        Convert the JDFTXOutfile object to a JDFTXInfile object with the most recent structure.
+        If the input structure is desired, simply fetch JDFTXOutfile.infile
+
+        Returns:
+            JDFTXInfile: A JDFTXInfile object representing the input parameters of the JDFTXOutfile.
+        """
+        return self.slices[-1].to_jdftxinfile()
+
     def __post_init__(self):
-        if len(self.slices):
+        last_slice = None
+        for slc in self.slices[::-1]:
+            if slc is not None:
+                last_slice = slc
+                break
+        if last_slice is not None:
             for var in _jof_atr_from_last_slice:
-                setattr(self, var, getattr(self.slices[-1], var))
+                setattr(self, var, getattr(last_slice, var))
             self.trajectory = self._get_trajectory()
 
     def _get_trajectory(self) -> Trajectory | None:
@@ -574,11 +749,11 @@ class JDFTXOutfile:
             if outfile_slice is not None:
                 if traj is None:
                     traj = outfile_slice.trajectory
-                else:
+                elif outfile_slice.trajectory is not None:
                     traj.extend(outfile_slice.trajectory)
         return traj
 
-    def to_dict(self) -> dict:
+    def as_dict(self) -> dict:
         """
         Convert the JDFTXOutfile object to a dictionary.
 
@@ -588,12 +763,16 @@ class JDFTXOutfile:
         dct = {}
         for fld in self.__dataclass_fields__:
             if fld == "slices":
-                dct[fld] = [slc.to_dict() for slc in self.slices]
+                dct[fld] = [slc.as_dict() for slc in self.slices]
                 continue
             value = getattr(self, fld)
             dct[fld] = value
         return dct
 
+    @deprecated(as_dict, deadline=(2025, 10, 4))
+    def to_dict(self):
+        return self.as_dict()
+
     ###########################################################################
     # Magic methods
     ###########################################################################
