@@ -26,11 +26,10 @@ from pymatgen.io.lammps.data import CombinedData, LammpsData
 from pymatgen.io.template import TemplateInputGen
 
 if TYPE_CHECKING:
-    from os import PathLike
-
     from typing_extensions import Self
 
     from pymatgen.io.core import InputSet
+    from pymatgen.util.typing import PathLike
 
 __author__ = "Kiran Mathew, Brandon Wood, Zhi Deng, Manas Likhit, Guillaume Brunin (Matgenix)"
 __copyright__ = "Copyright 2018, The Materials Virtual Lab"
@@ -347,10 +346,11 @@ class LammpsInputFile(InputFile):
         stages = self.stages[: indices_stages_to_merge[0]]
 
         merge_name = "Merge of: " + ", ".join([self.stages_names[i] for i in indices_stages_to_merge])
-        merged_commands = []
-        for i in indices_stages_to_merge:
-            for j in range(len(self.stages[i]["commands"])):
-                merged_commands.append(self.stages[i]["commands"][j])
+        merged_commands = [
+            self.stages[i]["commands"][j]
+            for i in indices_stages_to_merge
+            for j in range(len(self.stages[i]["commands"]))
+        ]
 
         merged_stages = {"stage_name": merge_name, "commands": merged_commands}
         stages.append(merged_stages)
@@ -540,7 +540,7 @@ class LammpsInputFile(InputFile):
 
     def write_file(
         self,
-        filename: str | PathLike,
+        filename: PathLike,
         ignore_comments: bool = False,
         keep_stages: bool = True,
     ) -> None:
@@ -829,11 +829,9 @@ class LammpsInputFile(InputFile):
         string_list = string_list[imin : imax + 1]
 
         # Get rid of empty comments that are there just for cosmetic reasons
-        new_list = []
-        for string in string_list:
-            if len(string) > 1 or not (len(string.strip()) == 1 and string[0] == "#"):
-                new_list.append(string)
-        string_list = new_list
+        string_list = [
+            string for string in string_list if len(string) > 1 or not (len(string.strip()) == 1 and string[0] == "#")
+        ]
 
         # Keep only a single empty lines when there are multiple ones
         lines = [string_list[0]]
@@ -1068,10 +1066,13 @@ def write_lammps_inputs(
         ... timestep        0.005
         ...
         ... run             $nsteps'''
+
         >>> write_lammps_inputs(".", eam_template, settings={"temperature": 1600.0, "nsteps": 100})
+
         >>> with open("in.lammps", encoding="utf-8") as file:
         ...     script = file.read()
-        >>> print(script)
+
+        >>> script
         units           metal
         atom_style      atomic
 
