@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import logging
 import os
 import tempfile
 from shutil import which
@@ -32,6 +33,8 @@ if TYPE_CHECKING:
 __author__ = "Kiran Mathew, Brandon Wood, Michael Humbert"
 __email__ = "kmathew@lbl.gov"
 
+logger = logging.getLogger(__name__)
+
 
 class Polymer:
     """
@@ -265,8 +268,7 @@ class PackmolRunner:
             input_dir (str): path to the input directory
         """
         with open(f"{input_dir}/{self.input_file}", mode="w", encoding="utf-8") as inp:
-            for key, val in self.control_params.items():
-                inp.write(f"{key} {self._format_param_val(val)}\n")
+            inp.writelines(f"{key} {self._format_param_val(val)}\n" for key, val in self.control_params.items())
             # write the structures of the constituent molecules to file and set
             # the molecule id and the corresponding filename in the packmol
             # input file.
@@ -287,8 +289,7 @@ class PackmolRunner:
 
                 inp.write("\n")
                 inp.write(f"structure {os.path.join(input_dir, str(idx))}.{self.control_params['filetype']}\n")
-                for key, val in self.param_list[idx].items():
-                    inp.write(f"  {key} {self._format_param_val(val)}\n")
+                inp.writelines(f"  {key} {self._format_param_val(val)}\n" for key, val in self.param_list[idx].items())
                 inp.write("end structure\n")
 
     def run(self, site_property: str | None = None) -> Molecule:
@@ -313,7 +314,7 @@ class PackmolRunner:
             if os.path.isfile(output_file):
                 packed_mol = BabelMolAdaptor.from_file(output_file, self.control_params["filetype"])
                 packed_mol = packed_mol.pymatgen_mol
-                print(f"packed molecule written to {self.control_params['output']}")
+                logger.info(f"packed molecule written to {self.control_params['output']}")
                 if site_property:
                     packed_mol = self.restore_site_properties(site_property=site_property, filename=output_file)
                 return packed_mol
@@ -458,7 +459,7 @@ class LammpsRunner:
     def run(self) -> tuple[bytes, bytes]:
         """Write the input/data files and run LAMMPS."""
         lammps_cmd = [*self.lammps_bin, "-in", self.input_filename]
-        print(f"Running: {' '.join(lammps_cmd)}")
+        logger.info(f"Running: {' '.join(lammps_cmd)}")
         with Popen(lammps_cmd, stdout=PIPE, stderr=PIPE) as process:
             stdout, stderr = process.communicate()
         return stdout, stderr
