@@ -139,7 +139,10 @@ class NwTask(MSONable):
         if NWCHEM_BASIS_LIBRARY is not None:
             for b in set(self.basis_set.values()):
                 if re.sub(r"\*", "s", b.lower()) not in NWCHEM_BASIS_LIBRARY:
-                    warnings.warn(f"Basis set {b} not in NWCHEM_BASIS_LIBRARY")
+                    warnings.warn(
+                        f"Basis set {b} not in NWCHEM_BASIS_LIBRARY",
+                        stacklevel=2,
+                    )
 
         self.basis_set_option = basis_set_option
 
@@ -154,13 +157,13 @@ class NwTask(MSONable):
         theory_spec = []
         if self.theory_directives:
             theory_spec.append(f"{self.theory}")
-            for k in sorted(self.theory_directives):
-                theory_spec.append(f" {k} {self.theory_directives[k]}")
+            theory_spec.extend(f" {k} {self.theory_directives[k]}" for k in sorted(self.theory_directives))
             theory_spec.append("end")
         for k in sorted(self.alternate_directives):
             theory_spec.append(k)
-            for k2 in sorted(self.alternate_directives[k]):
-                theory_spec.append(f" {k2} {self.alternate_directives[k][k2]}")
+            theory_spec.extend(
+                f" {k2} {self.alternate_directives[k][k2]}" for k2 in sorted(self.alternate_directives[k])
+            )
             theory_spec.append("end")
 
         t = Template(
@@ -374,13 +377,13 @@ class NwInput(MSONable):
         out = []
         if self.memory_options:
             out.append(f"memory {self.memory_options}")
-        for d in self.directives:
-            out.append(f"{d[0]} {d[1]}")
+
+        out.extend(f"{d[0]} {d[1]}" for d in self.directives)
         out.append("geometry " + " ".join(self.geometry_options))
         if self.symmetry_options:
             out.append(" symmetry " + " ".join(self.symmetry_options))
-        for site in self._mol:
-            out.append(f" {site.specie.symbol} {site.x} {site.y} {site.z}")
+
+        out.extend(f" {site.specie.symbol} {site.x} {site.y} {site.z}" for site in self._mol)
         out.append("end\n")
         for task in self.tasks:
             out.extend((str(task), ""))
@@ -391,7 +394,7 @@ class NwInput(MSONable):
         Args:
             filename (str): Filename.
         """
-        with zopen(filename, mode="w") as file:
+        with zopen(filename, mode="wt", encoding="utf-8") as file:
             file.write(str(self))
 
     def as_dict(self):
@@ -531,8 +534,8 @@ class NwInput(MSONable):
         Returns:
             NwInput object
         """
-        with zopen(filename) as file:
-            return cls.from_str(file.read())
+        with zopen(filename, mode="rt", encoding="utf-8") as file:
+            return cls.from_str(file.read())  # type:ignore[arg-type]
 
 
 class NwInputError(Exception):
@@ -554,7 +557,7 @@ class NwOutput:
         """
         self.filename = filename
 
-        with zopen(filename) as file:
+        with zopen(filename, mode="rt", encoding="utf-8") as file:
             data = file.read()
 
         chunks = re.split(r"NWChem Input Module", data)
