@@ -57,9 +57,7 @@ class Xr:
         for idx, site in enumerate(self.structure, start=1):
             output.append(f"{idx} {site.specie} {site.x:.4f} {site.y:.4f} {site.z:.4f}")
         mat = self.structure.lattice.matrix
-        for _ in range(2):
-            for j in range(3):
-                output.append(f"{mat[j][0]:.4f} {mat[j][1]:.4f} {mat[j][2]:.4f}")
+        output.extend(f"{mat[j][0]:.4f} {mat[j][1]:.4f} {mat[j][2]:.4f}" for _ in range(2) for j in range(3))
         return "\n".join(output)
 
     def write_file(self, filename: str | Path) -> None:
@@ -69,7 +67,7 @@ class Xr:
             filename (str): name of the file to write to.
         """
         with zopen(filename, mode="wt", encoding="utf-8") as file:
-            file.write(str(self) + "\n")
+            file.write(str(self) + "\n")  # type:ignore[arg-type]
 
     @classmethod
     def from_str(cls, string: str, use_cores: bool = True, thresh: float = 1e-4) -> Self:
@@ -156,4 +154,4 @@ class Xr:
                     file.
         """
         with zopen(filename, mode="rt", encoding="utf-8") as file:
-            return cls.from_str(file.read(), use_cores=use_cores, thresh=thresh)
+            return cls.from_str(file.read(), use_cores=use_cores, thresh=thresh)  # type:ignore[arg-type]
