@@ -8,7 +8,7 @@ from typing import TYPE_CHECKING
 from monty.json import MSONable
 
 if TYPE_CHECKING:
-    from typing import Any, Literal
+    from typing import Any
 
     from pymatgen.core import Structure
 
@@ -24,7 +24,7 @@ class AbstractTransformation(MSONable, abc.ABC):
     """Abstract transformation class."""
 
     @abc.abstractmethod
-    def apply_transformation(self, structure: Structure) -> Structure | list[dict[str, Any]]:
+    def apply_transformation(self, structure: Structure) -> Structure | list[dict[str, Any]] | list[Structure]:
         """Apply the transformation to a structure. Depending on whether a
         transformation is one-to-many, there may be an option to return a
         ranked list of structures.
@@ -57,7 +57,7 @@ class AbstractTransformation(MSONable, abc.ABC):
         """
 
     @property
-    def is_one_to_many(self) -> Literal[False]:
+    def is_one_to_many(self) -> bool:
         """Determine if a Transformation is a one-to-many transformation. In that case, the
         apply_transformation method should have a keyword arg "return_ranked_list" which
         allows for the transformed structures to be returned as a ranked list.
@@ -66,7 +66,7 @@ class AbstractTransformation(MSONable, abc.ABC):
         return False
 
     @property
-    def use_multiprocessing(self) -> Literal[False]:
+    def use_multiprocessing(self) -> bool:
         """Indicates whether the transformation can be applied by a
         subprocessing pool. This should be overridden to return True for
         transformations that the transmuter can parallelize.
