@@ -1,7 +1,6 @@
 from __future__ import annotations
 
-import json
-
+import orjson
 import pytest
 from pytest import approx
 
@@ -16,7 +15,7 @@ from pymatgen.analysis.chemenv.coordination_environments.chemenv_strategies impo
     SelfCSMNbSetWeight,
 )
 from pymatgen.analysis.chemenv.coordination_environments.structure_environments import StructureEnvironments
-from pymatgen.util.testing import TEST_FILES_DIR, PymatgenTest
+from pymatgen.util.testing import TEST_FILES_DIR, MatSciTest
 
 __author__ = "waroquiers"
 
@@ -39,7 +38,7 @@ class DummyVoronoiContainer:
     pass
 
 
-class StrategyWeights(PymatgenTest):
+class StrategyWeights(MatSciTest):
     def test_angle_weight(self):
         fake_nb_set = FakeNbSet()
         dummy_se = DummyStructureEnvironments()
@@ -240,8 +239,8 @@ class StrategyWeights(PymatgenTest):
 
     def test_self_csms_weight(self):
         # Get the StructureEnvironments for K2NaNb2Fe7Si8H4O31 (mp-743972)
-        with open(f"{struct_env_dir}/se_mp-743972.json", encoding="utf-8") as file:
-            dct = json.load(file)
+        with open(f"{struct_env_dir}/se_mp-743972.json", "rb") as file:
+            dct = orjson.loads(file.read())
         struct_envs = StructureEnvironments.from_dict(dct)
 
         # Get neighbors sets for which we get the weights
@@ -316,8 +315,8 @@ class StrategyWeights(PymatgenTest):
         assert abs(self_w - 0.14204073172729198) < 1e-8
 
         # Get the StructureEnvironments for SiO2 (mp-7000)
-        with open(f"{struct_env_dir}/se_mp-7000.json", encoding="utf-8") as file:
-            dct = json.load(file)
+        with open(f"{struct_env_dir}/se_mp-7000.json", "rb") as file:
+            dct = orjson.loads(file.read())
         struct_envs = StructureEnvironments.from_dict(dct)
 
         # Get neighbors sets for which we get the weights
@@ -360,8 +359,8 @@ class StrategyWeights(PymatgenTest):
 
     def test_delta_csms_weight(self):
         # Get the StructureEnvironments for K2NaNb2Fe7Si8H4O31 (mp-743972)
-        with open(f"{struct_env_dir}/se_mp-743972.json", encoding="utf-8") as file:
-            dct = json.load(file)
+        with open(f"{struct_env_dir}/se_mp-743972.json", "rb") as file:
+            dct = orjson.loads(file.read())
         struct_envs = StructureEnvironments.from_dict(dct)
 
         # Get neighbors sets for which we get the weights
@@ -492,8 +491,8 @@ class StrategyWeights(PymatgenTest):
         assert abs(delta_w - 0.103515625) < 1e-8
 
         # Get the StructureEnvironments for SiO2 (mp-7000)
-        with open(f"{struct_env_dir}/se_mp-7000.json", encoding="utf-8") as file:
-            dct = json.load(file)
+        with open(f"{struct_env_dir}/se_mp-7000.json", "rb") as file:
+            dct = orjson.loads(file.read())
         struct_envs = StructureEnvironments.from_dict(dct)
 
         # Get neighbors sets for which we get the weights
