@@ -5,13 +5,12 @@ import tarfile
 
 import numpy as np
 import pytest
-from monty.tempfile import ScratchDir
 from numpy.testing import assert_allclose, assert_array_equal
 
 from pymatgen.core.structure import Structure
 from pymatgen.io.abinit import EtsfReader
 from pymatgen.io.abinit.netcdf import AbinitHeader
-from pymatgen.util.testing import TEST_FILES_DIR, PymatgenTest
+from pymatgen.util.testing import TEST_FILES_DIR, MatSciTest
 
 try:
     import netCDF4
@@ -21,8 +20,8 @@ except ImportError:
 TEST_DIR = f"{TEST_FILES_DIR}/io/abinit"
 
 
-class TestEtsfReader(PymatgenTest):
-    def setUp(self):
+class TestEtsfReader(MatSciTest):
+    def setup_method(self):
         formulas = ["Si2"]
         self.GSR_paths = dct = {}
         for formula in formulas:
@@ -86,31 +85,42 @@ class TestEtsfReader(PymatgenTest):
 
     @pytest.mark.skipif(netCDF4 is None, reason="Requires Netcdf4")
     def test_read_fe(self):
-        with ScratchDir(".") as tmp_dir:
-            with tarfile.open(f"{TEST_DIR}/Fe_magmoms_collinear_GSR.tar.xz", mode="r:xz") as t:
-                t.extractall(tmp_dir)  # noqa: S202
-                ref_magmom_collinear = [-0.5069359730980665]
-                path = os.path.join(tmp_dir, "Fe_magmoms_collinear_GSR.nc")
+        with tarfile.open(f"{TEST_DIR}/Fe_magmoms_collinear_GSR.tar.xz", mode="r:xz") as t:
+            # TODO: remove attr check after only 3.12+
+            if hasattr(tarfile, "data_filter"):
+                t.extractall(".", filter="data")
+            else:
+                t.extractall(".")  # noqa: S202
 
-                with EtsfReader(path) as data:
-                    structure = data.read_structure()
-                    assert structure.site_properties["magmom"] == ref_magmom_collinear
+            ref_magmom_collinear = [-0.5069359730980665]
+            path = os.path.join(".", "Fe_magmoms_collinear_GSR.nc")
 
-            with tarfile.open(f"{TEST_DIR}/Fe_magmoms_noncollinear_GSR.tar.xz", mode="r:xz") as t:
-                t.extractall(tmp_dir)  # noqa: S202
-                ref_magmom_noncollinear = [[0.357939487, 0.357939487, 0]]
-                path = os.path.join(tmp_dir, "Fe_magmoms_noncollinear_GSR.nc")
+            # TODO: PR4128, EtsfReader would fail in Ubuntu CI with netCDF4 > 1.6.5
+            # Need someone with knowledge in netCDF4 to fix it
+            with EtsfReader(path) as data:
+                structure = data.read_structure()
+                assert structure.site_properties["magmom"] == ref_magmom_collinear
 
-                with EtsfReader(path) as data:
-                    structure = data.read_structure()
-                    assert structure.site_properties["magmom"] == ref_magmom_noncollinear
+        with tarfile.open(f"{TEST_DIR}/Fe_magmoms_noncollinear_GSR.tar.xz", mode="r:xz") as t:
+            # TODO: remove attr check after only 3.12+
+            if hasattr(tarfile, "data_filter"):
+                t.extractall(".", filter="data")
+            else:
+                t.extractall(".")  # noqa: S202
+            ref_magmom_noncollinear = [[0.357939487, 0.357939487, 0]]
+            path = os.path.join(".", "Fe_magmoms_noncollinear_GSR.nc")
+
+            with EtsfReader(path) as data:
+                structure = data.read_structure()
+                assert structure.site_properties["magmom"] == ref_magmom_noncollinear
 
 
-class TestAbinitHeader(PymatgenTest):
+class TestAbinitHeader(MatSciTest):
     def test_api(self):
         head = AbinitHeader(foo=1, bar=2)
         assert head.foo == 1
         assert str(head)
         assert head.to_str(verbose=2, title="title")
         # PLEASE DO NOT REMOVE THIS LINE AS THIS API HAS BEEN AROUND FOR SEVERAL YEARS,
-        assert head.to_string(verbose=2, title="title")
+        with pytest.warns(FutureWarning, match="to_string is deprecated"):
+            assert head.to_string(verbose=2, title="title")
