@@ -2,9 +2,9 @@ from __future__ import annotations
 
 import gzip
 import json
-from unittest import TestCase
 
 import numpy as np
+import orjson
 import pandas as pd
 import pytest
 from monty.json import MontyDecoder, MontyEncoder
@@ -14,14 +14,14 @@ from ruamel.yaml import YAML
 
 from pymatgen.core import Element, Lattice, Molecule, Structure
 from pymatgen.io.lammps.data import CombinedData, ForceField, LammpsBox, LammpsData, Topology, lattice_2_lmpbox
-from pymatgen.util.testing import TEST_FILES_DIR, PymatgenTest
+from pymatgen.util.testing import TEST_FILES_DIR, MatSciTest
 
 TEST_DIR = f"{TEST_FILES_DIR}/io/lammps"
 
 
-class TestLammpsBox(PymatgenTest):
+class TestLammpsBox(MatSciTest):
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         cls.peptide = LammpsBox(
             bounds=[
                 [36.840194, 64.211560],
@@ -73,9 +73,9 @@ class TestLammpsBox(PymatgenTest):
         )
 
 
-class TestLammpsData(PymatgenTest):
+class TestLammpsData(MatSciTest):
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         cls.peptide = LammpsData.from_file(filename=f"{TEST_DIR}/data.peptide")
         cls.ethane = LammpsData.from_file(filename=f"{TEST_DIR}/ethane.data")
         cls.quartz = LammpsData.from_file(filename=f"{TEST_DIR}/data.quartz", atom_style="atomic")
@@ -454,8 +454,8 @@ class TestLammpsData(PymatgenTest):
             "Angle Coeffs": [{"coeffs": [42.1845, 109.4712], "types": [("H", "O", "H")]}],
         }
         ff = ForceField(mass.items(), non_bond_coeffs, topo_coeffs)
-        with gzip.open(f"{TEST_DIR}/topologies_ice.json.gz") as file:
-            topo_dicts = json.load(file)
+        with gzip.open(f"{TEST_DIR}/topologies_ice.json.gz", "rb") as file:
+            topo_dicts = orjson.loads(file.read())
         topologies = [Topology.from_dict(d) for d in topo_dicts]
         box = LammpsBox([[-0.75694412, 44.165558], [0.38127473, 47.066074], [0.17900842, 44.193867]])
         ice = LammpsData.from_ff_and_topologies(box=box, ff=ff, topologies=topologies)
@@ -543,7 +543,7 @@ class TestLammpsData(PymatgenTest):
         assert pd.testing.assert_frame_equal(c2h6.topology[key], target_df) is None, key
 
 
-class TestTopology(TestCase):
+class TestTopology:
     def test_init(self):
         rng = np.random.default_rng()
         inner_charge = rng.random(10) - 0.5
@@ -654,7 +654,7 @@ class TestTopology(TestCase):
             [8, 1, 2, 3],
         ]
         assert_array_equal(tp_etoh["Dihedrals"], etoh_dihedrals)
-        assert json.dumps(topo_etoh.as_dict()) is not None
+        assert orjson.dumps(topo_etoh.as_dict()).decode() is not None
         # bond flag to off
         topo_etoh0 = Topology.from_bonding(molecule=etoh, bond=False, angle=True, dihedral=True)
         assert topo_etoh0.topologies is None
@@ -665,9 +665,9 @@ class TestTopology(TestCase):
         assert "Dihedrals" not in topo_etoh2.topologies
 
 
-class TestForceField(PymatgenTest):
+class TestForceField(MatSciTest):
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         mass_info = [
             ("A", "H"),
             ("B", Element("C")),
@@ -779,13 +779,13 @@ class TestForceField(PymatgenTest):
     def test_from_dict(self):
         dct = self.ethane.as_dict()
         json_str = json.dumps(dct)
-        decoded = ForceField.from_dict(json.loads(json_str))
+        decoded = ForceField.from_dict(orjson.loads(json_str))
         assert decoded.mass_info == self.ethane.mass_info
         assert decoded.nonbond_coeffs == self.ethane.nonbond_coeffs
         assert decoded.topo_coeffs == self.ethane.topo_coeffs
 
 
-class TestFunc(TestCase):
+class TestFunc:
     def test_lattice_2_lmpbox(self):
         rng = np.random.default_rng()
         matrix = np.diag(rng.integers(5, 14, size=(3,))) + rng.random((3, 3)) * 0.2 - 0.1
@@ -818,9 +818,9 @@ class TestFunc(TestCase):
         )
 
 
-class TestCombinedData(TestCase):
+class TestCombinedData:
     @classmethod
-    def setUpClass(cls):
+    def setup_class(cls):
         cls.ec = LammpsData.from_file(filename=f"{TEST_DIR}/ec.data.gz")
         cls.fec = LammpsData.from_file(filename=f"{TEST_DIR}/fec.data.gz")
         cls.li = LammpsData.from_file(filename=f"{TEST_DIR}/li.data")
@@ -957,7 +957,7 @@ class TestCombinedData(TestCase):
         assert ff["Pair Coeffs"].index[0] == 1
         assert ff["Bond Coeffs"].index[0] == 1
         assert ff["Angle Coeffs"].index[0] == 1
-        assert ff["Dihedral Coeffs"].index[0], 1
+        assert ff["Dihedral Coeffs"].index[0] == 1
         assert ff["Improper Coeffs"].index[0] == 1
         assert fec.atoms.index[0] == 1
         assert fec.atoms.loc[1, "molecule-ID"] == 1
