@@ -3,12 +3,12 @@ from __future__ import annotations
 from pytest import approx
 
 from pymatgen.io.pwmat.outputs import DosSpin, Movement, OutFermi, Report
-from pymatgen.util.testing import TEST_FILES_DIR, PymatgenTest
+from pymatgen.util.testing import TEST_FILES_DIR, MatSciTest
 
 TEST_DIR = f"{TEST_FILES_DIR}/io/pwmat"
 
 
-class TestMovement(PymatgenTest):
+class TestMovement(MatSciTest):
     def test_init_and_properties(self):
         filepath = f"{TEST_DIR}/MOVEMENT.lzma"
         movement = Movement(filepath)
@@ -28,14 +28,14 @@ class TestMovement(PymatgenTest):
         assert movement.ionic_steps[0]["e_tot"] == approx(-357677.2281)
 
 
-class TestOutFermi(PymatgenTest):
+class TestOutFermi(MatSciTest):
     def test_init_and_properties(self):
         filepath = f"{TEST_DIR}/OUT.FERMI.lzma"
         out_fermi = OutFermi(filepath)
         assert out_fermi.e_fermi == approx(-2.359)
 
 
-class TestReport(PymatgenTest):
+class TestReport(MatSciTest):
     def test_init_and_properties(self):
         filepath = f"{TEST_DIR}/REPORT"
         report = Report(filepath)
@@ -48,7 +48,7 @@ class TestReport(PymatgenTest):
         assert report.hsps == {}
 
 
-class TestDosSpin(PymatgenTest):
+class TestDosSpin(MatSciTest):
     def test_init_and_properties(self):
         filepath = f"{TEST_DIR}/DOS.spinup_projected"
         dos_spin = DosSpin(filepath)
