@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import copy
 import platform
-from unittest import TestCase
 
 import pytest
 from pytest import approx
@@ -17,8 +16,8 @@ from pymatgen.util.testing import TEST_FILES_DIR
 pybel = pytest.importorskip("openbabel.pybel")
 
 
-class TestBabelMolAdaptor(TestCase):
-    def setUp(self):
+class TestBabelMolAdaptor:
+    def setup_method(self):
         coords = [
             [0.000000, 0.000000, 0.000000],
             [0.000000, 0.000000, 1.089000],
@@ -116,7 +115,7 @@ class TestBabelMolAdaptor(TestCase):
         for site in opt_mol[1:]:
             assert site.distance(opt_mol[0]) == approx(1.09216, abs=1e-1)
 
-    @pytest.mark.skipif(platform.system() == "Windows", reason="Tests for openbabel failing on Win")
+    @pytest.mark.xfail(platform.system() == "Windows", reason="Tests for openbabel failing on Win")
     def test_confab_conformers(self):
         mol = pybel.readstring("smi", "CCCC").OBMol
         adaptor = BabelMolAdaptor(mol)
