@@ -5,24 +5,18 @@ import pytest
 from pytest import approx
 
 from pymatgen.analysis.structure_matcher import StructureMatcher
-from pymatgen.core import Composition, DummySpecies, Element, Lattice, Species, Structure
+from pymatgen.core import Composition, DummySpecies, Element, Lattice, Species, Structure, SymmOp
 from pymatgen.electronic_structure.core import Magmom
 from pymatgen.io.cif import CifBlock, CifParser, CifWriter
 from pymatgen.symmetry.structure import SymmetrizedStructure
-from pymatgen.util.testing import TEST_FILES_DIR, VASP_IN_DIR, PymatgenTest
-
-try:
-    import pybtex
-except ImportError:
-    pybtex = None
-
+from pymatgen.util.testing import TEST_FILES_DIR, VASP_IN_DIR, MatSciTest
 
 MCIF_TEST_DIR = f"{TEST_FILES_DIR}/io/cif/mcif"
 
 
-class TestCifBlock(PymatgenTest):
+class TestCifBlock(MatSciTest):
     def test_to_str(self):
-        with open(f"{TEST_FILES_DIR}/cif/Graphite.cif") as file:
+        with open(f"{TEST_FILES_DIR}/cif/Graphite.cif", encoding="utf-8") as file:
             cif_str = file.read()
         cif_block = CifBlock.from_str(cif_str)
         cif_str_2 = str(CifBlock.from_str(str(cif_block)))
@@ -164,7 +158,7 @@ loop_
         assert str(CifBlock(data, loops, "test")) == cif_str
 
 
-class TestCifIO(PymatgenTest):
+class TestCifIO(MatSciTest):
     def test_cif_parser(self):
         parser = CifParser(f"{TEST_FILES_DIR}/cif/LiFePO4.cif")
         for struct in parser.parse_structures():
@@ -197,7 +191,7 @@ class TestCifIO(PymatgenTest):
         for struct in parser.parse_structures():
             assert struct.formula == "Li20.2 Ge2.06 P3.94 S24", "Incorrectly parsed cif."
 
-        with open(f"{TEST_FILES_DIR}/cif/FePO4.cif") as cif_file:
+        with open(f"{TEST_FILES_DIR}/cif/FePO4.cif", encoding="utf-8") as cif_file:
             cif_str = cif_file.read()
 
         parser = CifParser.from_str(cif_str)
@@ -219,6 +213,18 @@ class TestCifIO(PymatgenTest):
         assert len(parser.parse_structures(primitive=False)[0]) == 2
         assert not parser.has_errors
 
+    def test_parse_bad_superflat(self):
+        """
+        Test unphysically "flat" structure with volume near zero,
+        which would originally lead to infinite loop (PR4133).
+        """
+        parser = CifParser(f"{TEST_FILES_DIR}/cif/bad_superflat_inf_loop.cif.gz")
+        with (
+            pytest.raises(ValueError, match="Invalid CIF file with no structures"),
+            pytest.warns(UserWarning, match="Å below threshold, double check your structure."),
+        ):
+            parser.parse_structures()
+
     def test_get_symmetrized_structure(self):
         parser = CifParser(f"{TEST_FILES_DIR}/cif/Li2O.cif")
         sym_structure = parser.parse_structures(primitive=False, symmetrized=True)[0]
@@ -231,6 +237,22 @@ class TestCifIO(PymatgenTest):
         ]
         assert set(sym_structure.labels) == {"O1", "Li1"}
 
+    def test_no_sym_ops(self):
+        with open(f"{TEST_FILES_DIR}/cif/Li2O.cif") as fin, open("test.cif", "w") as fout:
+            for line_num, line in enumerate(fin, start=1):
+                # Skip "_symmetry_equiv_pos_as_xyz", "_symmetry_space_group_name_H-M"
+                # and "_symmetry_Int_Tables_number" sections such that
+                # no symmop info would be available
+                if line_num not in range(44, 236) and line_num not in {40, 41}:
+                    fout.write(line)
+
+        parser = CifParser("test.cif")
+
+        # Need `parse_structures` call to update `symmetry_operations`
+        _structure = parser.parse_structures(primitive=False, symmetrized=False)[0]
+        assert parser.symmetry_operations[0] == SymmOp.from_xyz_str("x, y, z")
+        assert any("No _symmetry_equiv_pos_as_xyz type key found" in msg for msg in parser.warnings)
+
     def test_site_symbol_preference(self):
         parser = CifParser(f"{TEST_FILES_DIR}/cif/site_type_symbol_test.cif")
         assert parser.parse_structures()[0].formula == "Ge1.6 Sb1.6 Te4"
@@ -364,7 +386,7 @@ class TestCifIO(PymatgenTest):
         # Symbol in capital letters
         parser = CifParser(f"{TEST_FILES_DIR}/cif/Cod_2100513.cif")
         for struct in parser.parse_structures():
-            assert struct.formula == "Ca4 Nb2.0 Al2 O12"
+            assert struct.formula == "Ca4 Nb2 Al2 O12"
 
         # Label in capital letters
         parser = CifParser(f"{TEST_FILES_DIR}/cif/Cod_4115344.cif")
@@ -759,7 +781,7 @@ loop_
 
     def test_not_check_occu(self):
         # Test large occupancy with check_occu turned off
-        with open(f"{TEST_FILES_DIR}/cif/site_type_symbol_test.cif") as cif_file:
+        with open(f"{TEST_FILES_DIR}/cif/site_type_symbol_test.cif", encoding="utf-8") as cif_file:
             cif_str = cif_file.read()
         cif_str = cif_str.replace("Te    Te 1.0000", "Te_label    Te 10.0", 1)
 
@@ -793,12 +815,10 @@ loop_
     def test_replacing_finite_precision_frac_coords(self):
         cif = f"{TEST_FILES_DIR}/cif/cif_finite_precision_frac_coord_error.cif"
         parser = CifParser(cif)
-        with pytest.warns(UserWarning) as record:
-            struct = parser.parse_structures()[0]
-
-        assert len(record) == 3
         warn_msg = "4 fractional coordinates rounded to ideal values to avoid issues with finite precision."
-        assert warn_msg in str(record[-1])
+        with pytest.warns(UserWarning, match=warn_msg) as record:
+            struct = parser.parse_structures()[0]
+        assert len(record) == 3
 
         assert str(struct.composition) == "N5+72"
         assert warn_msg in parser.warnings
@@ -873,7 +893,7 @@ Si1 Si 0 0 0 1 0.0
             parser.parse_structures()
 
     def test_no_check_occu(self):
-        with open(f"{TEST_FILES_DIR}/cif/site_type_symbol_test.cif") as cif_file:
+        with open(f"{TEST_FILES_DIR}/cif/site_type_symbol_test.cif", encoding="utf-8") as cif_file:
             cif_str = cif_file.read()
         cif_str = cif_str.replace("Te    Te 1.0000", "Te    Te 1.5000", 1)
 
@@ -884,7 +904,7 @@ Si1 Si 0 0 0 1 0.0
         for tol in (1.5, 10):
             parser = CifParser.from_str(cif_str, occupancy_tolerance=tol)
             structs = parser.parse_structures(primitive=False, check_occu=False)[0]
-            assert structs[0].species.as_dict()["Te"] == 1.5
+            assert structs[0].species.as_dict()["Te"] == approx(1.5)
 
     def test_cif_writer_write_file(self):
         struct1 = Structure.from_file(f"{VASP_IN_DIR}/POSCAR")
@@ -896,7 +916,7 @@ Si1 Si 0 0 0 1 0.0
 
         # test write_file append mode='a'
         struct2 = Structure.from_file(f"{TEST_FILES_DIR}/cif/Graphite.cif")
-        CifWriter(struct2).write_file(out_path, mode="a")
+        CifWriter(struct2).write_file(out_path, mode="at")
 
         read_structs = CifParser(out_path).parse_structures()
         assert len(read_structs) == 2
@@ -910,7 +930,7 @@ Si1 Si 0 0 0 1 0.0
 
     def test_missing_elements(self):
         cif_str = ""
-        with open(f"{TEST_FILES_DIR}/cif/MgNiF6.cif") as file:
+        with open(f"{TEST_FILES_DIR}/cif/MgNiF6.cif", encoding="utf-8") as file:
             for line in file:
                 if "_chemical_formula_sum" in line:
                     # remove this line
@@ -928,7 +948,7 @@ Si1 Si 0 0 0 1 0.0
 
     def test_incorrect_stoichiometry(self):
         cif_str = ""
-        with open(f"{TEST_FILES_DIR}/cif/MgNiF6.cif") as file:
+        with open(f"{TEST_FILES_DIR}/cif/MgNiF6.cif", encoding="utf-8") as file:
             for line in file:
                 if "_chemical_formula_sum" in line:
                     line = line.replace("F6", "F5")
@@ -940,12 +960,12 @@ Si1 Si 0 0 0 1 0.0
         assert "Incorrect stoichiometry" in failure_reason
 
     def test_missing_cif_composition(self):
-        with open(f"{TEST_FILES_DIR}/cif/LiFePO4.cif") as file:
+        with open(f"{TEST_FILES_DIR}/cif/LiFePO4.cif", encoding="utf-8") as file:
             cif_str = file.read()
         # remove only key that gives info about CIF composition in this file
         cif_str = "\n".join([line for line in cif_str.split("\n") if "_atom_site_type_symbol" not in line])
         test_cif_file = f"{self.tmp_path}/test_broken.cif"
-        with open(test_cif_file, "w+") as file:
+        with open(test_cif_file, "w+", encoding="utf-8") as file:
             file.write(cif_str)
 
         cif = CifParser(test_cif_file)
@@ -953,11 +973,11 @@ Si1 Si 0 0 0 1 0.0
         assert failure_reason == "Cannot determine chemical composition from CIF! 'NoneType' object is not iterable"
 
     def test_invalid_cif_composition(self):
-        with open(f"{TEST_FILES_DIR}/cif/LiFePO4.cif") as file:
+        with open(f"{TEST_FILES_DIR}/cif/LiFePO4.cif", encoding="utf-8") as file:
             cif_str = file.read()
 
         test_cif_file = f"{self.tmp_path}/test_broken.cif"
-        with open(test_cif_file, "w+") as file:
+        with open(test_cif_file, "w+", encoding="utf-8") as file:
             # replace Li with dummy atom X
             file.write(cif_str.replace("Li", "X"))
 
@@ -980,15 +1000,15 @@ Si1 Si 0 0 0 1 0.0
         struct.add_site_property(label := "hello", [1.0] * (len(struct) - 1) + [-1.0])
         out_path = f"{self.tmp_path}/test2.cif"
         CifWriter(struct, write_site_properties=True).write_file(out_path)
-        with open(out_path) as file:
+        with open(out_path, encoding="utf-8") as file:
             cif_str = file.read()
         assert f"_atom_site_occupancy\n _atom_site_{label}\n" in cif_str
         assert "Fe  Fe0  1  0.21872822  0.75000000  0.47486711  1  1.0" in cif_str
         assert "O  O23  1  0.95662769  0.25000000  0.29286233  1  -1.0" in cif_str
 
 
-class TestMagCif(PymatgenTest):
-    def setUp(self):
+class TestMagCif(MatSciTest):
+    def setup_method(self):
         self.mcif = CifParser(f"{MCIF_TEST_DIR}/magnetic.example.NiO.mcif")
         self.mcif_ncl = CifParser(f"{MCIF_TEST_DIR}/magnetic.ncl.example.GdB4.mcif")
         self.mcif_incommensurate = CifParser(f"{MCIF_TEST_DIR}/magnetic.incommensurate.example.Cr.mcif")
@@ -1065,7 +1085,7 @@ Gd1 5.05 5.05 0.0"""
         assert s_ncl.matches(s_ncl_from_msg)
 
     def test_write(self):
-        with open(f"{MCIF_TEST_DIR}/GdB4-writer-ref.mcif") as file:
+        with open(f"{MCIF_TEST_DIR}/GdB4-writer-ref.mcif", encoding="utf-8") as file:
             cw_ref_str = file.read()
         s_ncl = self.mcif_ncl.parse_structures(primitive=False)[0]
 
@@ -1085,14 +1105,14 @@ Gd1 5.05 5.05 0.0"""
         s_ncl.add_site_property("magmom", float_magmoms)
         cw = CifWriter(s_ncl, write_magmoms=True)
 
-        with open(f"{MCIF_TEST_DIR}/GdB4-str-magnitudes-ref.mcif") as file:
+        with open(f"{MCIF_TEST_DIR}/GdB4-str-magnitudes-ref.mcif", encoding="utf-8") as file:
             cw_ref_str_magnitudes = file.read()
 
         assert str(cw).strip() == cw_ref_str_magnitudes.strip()
         # test we're getting correct magmoms in ncl case
         s_ncl2 = self.mcif_ncl2.parse_structures()[0]
         list_magmoms = [list(m) for m in s_ncl2.site_properties["magmom"]]
-        assert list_magmoms[0][0] == 0.0
+        assert list_magmoms[0][0] == approx(0.0)
         assert list_magmoms[0][1] == approx(5.9160793408726366)
         assert list_magmoms[1][0] == approx(-5.1234749999999991)
         assert list_magmoms[1][1] == approx(2.9580396704363183)
@@ -1103,13 +1123,12 @@ Gd1 5.05 5.05 0.0"""
         cw = CifWriter(s_manual, write_magmoms=True)
 
         # check oxidation state
-        with open(f"{MCIF_TEST_DIR}/CsCl-manual-oxi-ref.mcif") as file:
+        with open(f"{MCIF_TEST_DIR}/CsCl-manual-oxi-ref.mcif", encoding="utf-8") as file:
             cw_manual_oxi_string = file.read()
         s_manual.add_oxidation_state_by_site([1, 1])
         cw = CifWriter(s_manual, write_magmoms=True)
         assert str(cw) == cw_manual_oxi_string
 
-    @pytest.mark.skipif(pybtex is None, reason="pybtex not present")
     def test_bibtex(self):
         ref_bibtex_string = """@article{cifref0,
     author = "Blanco, J.A.",
